/*********************************************************
	SBITMAP.CPP -- Implementation a simple bitmap class.
					 (Resource DDB's only in this version).
*********************************************************/
#include "sbitmap.h"

SBitmap::SBitmap(void)
{
	hbm = 0;
	cx = 0;
	cy = 0;
	dwRasterOp = SRCCOPY;
}


SBitmap::SBitmap(HINSTANCE hInst, LPSTR lpszBitmapResource, DWORD dwDefaultRasterOp)
{
	hbm = 0;
	BITMAP bm;
	cx = 0;
	cy = 0;
	hbm = LoadBitmap(hInst, lpszBitmapResource);
	if(hbm)
	{
		GetObject(hbm, sizeof(BITMAP), &bm);
		cx = bm.bmWidth;
		cy = bm.bmHeight;
	}
	else
	{
		char szMsg[256];
		wsprintf(szMsg, "Unable to load resource bitmap '%s'.", lpszBitmapResource);
		MessageBox(NULL, szMsg, "Can't Load Bitmap", MB_ICONSTOP);
	}
	dwRasterOp = dwDefaultRasterOp;
}


SBitmap::SBitmap(const SBitmap& bmOld)
{
	hbm = bmOld.hbm;

	cx = bmOld.cx;
	cy = bmOld.cy;

	dwRasterOp = bmOld.dwRasterOp;
}


// Use screen to create a blank bitmap.
SBitmap::SBitmap(HDC hdc, int cxNew, int cyNew, DWORD dwDefaultRasterOp)
{
	cx = 0;
	cy = 0;

	hbm = CreateCompatibleBitmap(hdc, cxNew, cyNew);

	if(hbm)
	{
		cx = cxNew;
		cy = cyNew;
	}
	else
	{
		char szMsg[256];
		wsprintf(szMsg, "Unable to create blank bitmap for %d x %d pixels.", cx, cy);
		MessageBox(NULL, szMsg, "Can't Create Bitmap", MB_ICONSTOP);
	}
	dwRasterOp = dwDefaultRasterOp;
}


// Create a bitmap using an area of a screen.)
SBitmap::SBitmap(HDC hdc, LPRECT pRect, DWORD dwDefaultRasterOp)
{
	HDC hdcMem = CreateCompatibleDC(hdc);
	HBITMAP hbmOld;

	cx = pRect->right - pRect->left;
	cy = pRect->bottom - pRect->top;

	hbmOld = CreateCompatibleBitmap(hdc, cx, cy);
	if(hbm)
	{
		hbmOld = SelectObject(hdcMem, hbm);
		BitBlt(hdcMem, 0, 0, cx, cy,
				 hdc, 0, 0, SRCCOPY);
	}
	else
	{
		cx = 0;
		cy = 0;
		char szMsg[256];
		wsprintf(szMsg, "Unable to create blank bitmap for %d x %d pixels.", cx, cy);
		MessageBox(NULL, szMsg, "Can't Create Bitmap", MB_ICONSTOP);
	}

	SelectObject(hdc, hbmOld);

	DeleteDC(hdcMem);

	dwRasterOp = dwDefaultRasterOp;
}


SBitmap& SBitmap::operator=(const SBitmap& bmOld)
{
	if(this != &bmOld)
	{
		hbm = bmOld.hbm;

		cx = bmOld.cx;
		cy = bmOld.cy;

		dwRasterOp = bmOld.dwRasterOp;
	}
	return *this;
}

