/*********************************************************
	SCARTOON.CPP -- 	Implementation of a "cartoon" class.
							Manages a series of bitmaps to provide
							spritee animation.
*********************************************************/
#include "scartoon.h"
#include "memory.h"

void SetBitmapFrame(BITMAPFRAME& ThisFrame,
						  int nDirection, int nFrame,
						  LPSTR lpszXORBitmapResource,
						  LPSTR lpszANDBitmapResource,
						  int cxNewDelta, int cyNewDelta,
						  float fNewMagnifyFactor,
						  int cxNew, int cyNew)
{
	ThisFrame.nDirection = nDirection;
	ThisFrame.nFrame = nFrame;

	ThisFrame.cx = cxNew;
	ThisFrame.cy = cyNew;

	ThisFrame.cxDelta = cxNewDelta;
	ThisFrame.cyDelta = cyNewDelta;

	ThisFrame.fMagnifyFactor = fNewMagnifyFactor;

	ThisFrame.hbmXOR = NULL;
	ThisFrame.hbmAND = NULL;

	lstrcpy(ThisFrame.szXORBmpName, lpszXORBitmapResource);
	lstrcpy(ThisFrame.szANDBmpName, lpszANDBitmapResource);

	ThisFrame.bLoadedOk = FALSE;
}


SCartoon::SCartoon(int nNewMaxDirections, int nNewMaxFrames,
							BITMAPFRAME *pFrames, int nFrames,
							HWND hwndNewFrame, int bLoadResourcesToo)
{
	int rc;

	bInitOk = FALSE;

	x = 0;
	y = 0;

	fMagnify = 1.0;
	bUseMagnification = FALSE;

	bInMotion = TRUE;  // We're moving by default.

	nCurrentDirection = -1;
	nCurrentFrame = -1;

	nMaxDirections = nNewMaxDirections;
	nMaxFrames = nNewMaxFrames;

	BmpFrames = NULL;
	nBmpFramesCount = 0;

	// Save our window.
	hwndFrame = hwndNewFrame;

	if(hwndFrame)
		GetClientRect(hwndFrame, &rWin);
	else
		SetRect(&rWin, -1, -1, -1, -1);

	if(pFrames && nFrames)
	{
		rc = AddBitmapFramesToArray(pFrames, nFrames, bLoadResourcesToo, hwndFrame);
		SortBitmapFramesArray();
	}

	if(bLoadResourcesToo && hwndFrame)
	{
		rc = LoadResourcesInFramesArray(hwndFrame);
		bInitOk = rc;
	}
	rc; // Kill warning.
}

// Copy constructor
SCartoon::SCartoon(const SCartoon& OldCartoon)
{
	// Copy old SCartoon...
	x = OldCartoon.x;
	y = OldCartoon.y;

	fMagnify = OldCartoon.fMagnify;
	bUseMagnification = OldCartoon.bUseMagnification;

	bInMotion = OldCartoon.bInMotion;

	nCurrentDirection = OldCartoon.nCurrentDirection;
	nCurrentFrame = OldCartoon.nCurrentFrame;

	nMaxDirections = OldCartoon.nMaxDirections;
	nMaxFrames = OldCartoon.nMaxFrames;

	if(OldCartoon.BmpFrames && OldCartoon.nBmpFramesCount)
	{
		nBmpFramesCount = OldCartoon.nBmpFramesCount;
		BmpFrames = new BITMAPFRAME[nBmpFramesCount];
		if(BmpFrames)
		{
			memcpy(BmpFrames, OldCartoon.BmpFrames, sizeof(BITMAPFRAME) * nBmpFramesCount);
		}
	}
	else
	{
		BmpFrames = NULL;
		nBmpFramesCount = OldCartoon.nBmpFramesCount;
	}

	CopyRect(&rWin, &OldCartoon.rWin);
	hwndFrame = OldCartoon.hwndFrame;

	bInitOk = OldCartoon.bInitOk;
}


// Assignment operator
SCartoon& SCartoon::operator=(const SCartoon& OldCartoon)
{
	if(this != &OldCartoon)
	{
		// Copy old SCartoon...
		x = OldCartoon.x;
		y = OldCartoon.y;

		fMagnify = OldCartoon.fMagnify;
		bUseMagnification = OldCartoon.bUseMagnification;

		bInMotion = OldCartoon.bInMotion;

		nCurrentDirection = OldCartoon.nCurrentDirection;
		nCurrentFrame = OldCartoon.nCurrentFrame;

		nMaxDirections = OldCartoon.nMaxDirections;
		nMaxFrames = OldCartoon.nMaxFrames;

		if(BmpFrames)
		{
			delete[] BmpFrames;
		}

		if(OldCartoon.BmpFrames && OldCartoon.nBmpFramesCount)
		{
			nBmpFramesCount = OldCartoon.nBmpFramesCount;
			BmpFrames = new BITMAPFRAME[nBmpFramesCount];
			if(BmpFrames)
			{
				memcpy(BmpFrames, OldCartoon.BmpFrames, sizeof(BITMAPFRAME) * nBmpFramesCount);
			}
		}
		else
		{
			BmpFrames = NULL;
			nBmpFramesCount = OldCartoon.nBmpFramesCount;
		}

		CopyRect(&rWin, &OldCartoon.rWin);
		hwndFrame = OldCartoon.hwndFrame;

		bInitOk = OldCartoon.bInitOk;
	}
	return *this;
}


SCartoon::~SCartoon(void)
{
	// Clean-up any loaded bitmap resources.
	CleanupResourcesInFramesArray();

	if(BmpFrames)
		delete[] BmpFrames;
}

// Add entries to BmpFrames Array
BOOL SCartoon::AddBitmapFramesToArray(BITMAPFRAME *pFrames, int nFrames, int bLoadResourcesToo, HWND hwndMyFrame)
{
	if(!pFrames || !nFrames)
		return FALSE;  // Nothing to add.

	// Get space for new frames array
	BITMAPFRAME *pNewFrames = new BITMAPFRAME[nBmpFramesCount + nFrames];
	if(!pNewFrames)
	{
		MessageBox(NULL, "Can't allocate memory for bitmaps for SCartoon object!.", "Not Enought Memory", MB_ICONSTOP);
		return FALSE;
	}

	// Copy existing data into new array.
	if(nBmpFramesCount)
		memcpy(pNewFrames, BmpFrames, sizeof(BITMAPFRAME) * nBmpFramesCount);

	memcpy(&pNewFrames[nBmpFramesCount], pFrames, sizeof(BITMAPFRAME) * nFrames);

	// Make new array the old array.
	if(BmpFrames)
		delete[] BmpFrames;

	BmpFrames = pNewFrames;

	// Update total of frames in array.
	nBmpFramesCount = nBmpFramesCount + nFrames;

	if(bLoadResourcesToo)
	{
		// Try to load in bitmaps, if asked...
		if(hwndMyFrame)
			bInitOk = LoadResourcesInFramesArray(hwndMyFrame);
		else
			bInitOk = LoadResourcesInFramesArray(hwndFrame);
		return bInitOk;
	}


	return TRUE;
}

BOOL SCartoon::AddBitmapFrameToArray(int nDirection, int nFrame,
										LPSTR lpszXORBitmapResource,
										LPSTR lpszANDBitmapResource,
										int cxNewDelta, int cyNewDelta,
										float fNewMagnifyFactor,
										int cxNew, int cyNew,
										int bLoadResourcesToo, HWND hwndMyFrame)
{
	BITMAPFRAME NewBmpFrame;

	SetBitmapFrame(NewBmpFrame,
						nDirection, nFrame,
						lpszXORBitmapResource,
						lpszANDBitmapResource,
						cxNewDelta, cyNewDelta,
						fNewMagnifyFactor,
						cxNew, cyNew);

	return AddBitmapFramesToArray(&NewBmpFrame, 1, bLoadResourcesToo, hwndMyFrame);
}

void SCartoon::SortBitmapFramesArray(void)
{
	// Sort 'cels' by direction and frame count.
	BITMAPFRAME BmpFrameTemp;

	char szMsg[256];

	int bSomethingSwitched = TRUE;

	int bWarned = FALSE;

	int nCompareVal;

	while(bSomethingSwitched)
	{
		bSomethingSwitched = FALSE;
		for(int i = 0; i < nBmpFramesCount - 1; i++)
		{
			if(BmpFrames[i].nDirection > BmpFrames[i + 1].nDirection)
				nCompareVal = 1;
			else if(BmpFrames[i].nDirection == BmpFrames[i + 1].nDirection)
			{
				if(BmpFrames[i].nFrame > BmpFrames[i + 1].nFrame)
					nCompareVal = 1;
				else if(BmpFrames[i].nFrame < BmpFrames[i + 1].nFrame)
					nCompareVal = -1;
				else
				{
					if(!bWarned)
					{
						// Then two 'cels' have same direction and frame number! Generate warning.
						wsprintf(szMsg, "Warning: Duplicate 'Cels' found in SCartoon::SortBitmapFramesArray()! (Direction #%d, Frame #%d)", BmpFrames[i].nDirection, BmpFrames[i].nFrame);
						MessageBox(NULL, szMsg, "Duplicate Cels Found", MB_ICONSTOP);
						bWarned = TRUE;
					}
				}
			}
			else
				nCompareVal = -1;

			if(nCompareVal == 1)
			{
				// Then switch adjacent entries.
				memcpy(&BmpFrameTemp, &BmpFrames[i], sizeof(BITMAPFRAME));
				memcpy(&BmpFrames[i], &BmpFrames[i + 1], sizeof(BITMAPFRAME));
				memcpy(&BmpFrames[i + 1], &BmpFrameTemp, sizeof(BITMAPFRAME));
				bSomethingSwitched = TRUE;
			}
		}
	}
	return;
}

BOOL SCartoon::LoadResourcesInFramesArray(HWND hwnd)
{
	char szMsg[256];
	int bLoadOk;

	HBITMAP hbm;
	BITMAP bm;

	int bNoErrors = TRUE;

	HINSTANCE hInst;
#if defined (WIN32)
	hInst = (HINSTANCE)GetWindowLong(hwnd, GWL_HINSTANCE);
#else
	hInst = GetWindowWord(hwnd, GWW_HINSTANCE);
#endif

	for(int i = 0; i < nBmpFramesCount; i++)
	{
		bLoadOk = TRUE;

		hbm = LoadBitmap(hInst, BmpFrames[i].szXORBmpName);
		if(!hbm)
		{
			wsprintf(szMsg, "Can't load (XOR) bitmap resource '%s'", BmpFrames[i].szXORBmpName);
			MessageBox(NULL, szMsg, "Bitmap Load Failed", MB_ICONEXCLAMATION);

			BmpFrames[i].hbmXOR = NULL;
			bLoadOk = FALSE;
			bNoErrors = FALSE;
		}
		else
		{
			BmpFrames[i].hbmXOR = hbm;
		}

		hbm = LoadBitmap(hInst, BmpFrames[i].szANDBmpName);
		if(!hbm)
		{
			wsprintf(szMsg, "Can't load (AND) bitmap resource '%s'", BmpFrames[i].szXORBmpName);
			MessageBox(NULL, szMsg, "Bitmap Load Failed", MB_ICONEXCLAMATION);

			BmpFrames[i].hbmAND = NULL;
			bLoadOk = FALSE;
			bNoErrors = FALSE;
		}
		else
		{
			BmpFrames[i].hbmAND = hbm;
		}

		if(bLoadOk)
		{
			// Measure XOR bitmap.
			GetObject(BmpFrames[i].hbmXOR, sizeof(BITMAP), (LPSTR)&bm);
			BmpFrames[i].cx = bm.bmWidth;
			BmpFrames[i].cy = bm.bmHeight;
		}
		BmpFrames[i].bLoadedOk = bLoadOk;
	}
	return bNoErrors;
}

void SCartoon::CleanupResourcesInFramesArray(void)
{
	for(int i = 0; i < nBmpFramesCount; i++)
	{
		if(BmpFrames[i].hbmXOR)
		{
			DeleteObject(BmpFrames[i].hbmXOR);
			BmpFrames[i].hbmXOR = NULL;
		}
		if(BmpFrames[i].hbmAND)
		{
			DeleteObject(BmpFrames[i].hbmAND);
			BmpFrames[i].hbmAND = NULL;
		}
		BmpFrames[i].bLoadedOk = FALSE;
	}
}

// Outputs a rectangle with our cartoon sprite's current coordinates,
// (including magnification, if we're using it).
void SCartoon::GetCurrentCartoonRect(RECT& rDim,  int *pcxOriginalSize, int *pcyOriginalSize)
{
	if(!BmpFrames)
		return;

	BITMAPFRAME& ThisFrame = BmpFrames[nCurrentDirection * nMaxFrames + nCurrentFrame];

	if(ThisFrame.hbmXOR)
		SetRect(&rDim, x, y,
				  x + (int)((float)ThisFrame.cx * fMagnify),
				  y + (int)((float)ThisFrame.cy * fMagnify));
	else
		SetRect(&rDim, 0, 0, 0, 0);

	if(pcxOriginalSize)
		*pcxOriginalSize = ThisFrame.cx;
	if(pcyOriginalSize)
		*pcyOriginalSize = ThisFrame.cy;
}

HBITMAP SCartoon::GetCurrentBitmap(RECT *prDim, int *pcxOriginalSize, int *pcyOriginalSize)
{
	if(BmpFrames)
	{
		BITMAPFRAME& ThisFrame = BmpFrames[nCurrentDirection * nMaxFrames + nCurrentFrame];

		if(prDim)
		{
			if(ThisFrame.hbmXOR)
				SetRect(prDim, x, y,
						  x + (int)((float)ThisFrame.cx * fMagnify),
						  y + (int)((float)ThisFrame.cy * fMagnify));
			else
				SetRect(prDim, 0, 0, 0, 0);
		}
		if(pcxOriginalSize)
			*pcxOriginalSize = ThisFrame.cx;
		if(pcyOriginalSize)
			*pcyOriginalSize = ThisFrame.cy;
		return ThisFrame.hbmXOR;
	}
	else
		return NULL;
}

HBITMAP SCartoon::GetCurrentBitmapMask(RECT *prDim, int *pcxOriginalSize, int *pcyOriginalSize)
{
	if(BmpFrames)
	{
		BITMAPFRAME& ThisFrame = BmpFrames[nCurrentDirection * nMaxFrames + nCurrentFrame];

		if(prDim)
		{
			if(ThisFrame.hbmAND)
				SetRect(prDim, x, y,
						  x + (int)((float)ThisFrame.cx * fMagnify),
						  y + (int)((float)ThisFrame.cy * fMagnify));
			else
				SetRect(prDim, 0, 0, 0, 0);
		}
		if(pcxOriginalSize)
			*pcxOriginalSize = ThisFrame.cx;
		if(pcyOriginalSize)
			*pcyOriginalSize = ThisFrame.cy;
		return ThisFrame.hbmAND;
	}
	else
		return NULL;
}

SBitmap *SCartoon::GetCurrentSBitmap(SBitmap *pSBitmap, RECT *prDim)
{
	// Returns a pointer to an SBitmap object.
	// (Create one if we need to.)
	SBitmap *pOutSBitmap;

	if(!pSBitmap)
	{
		pOutSBitmap = new SBitmap;
		if(!pOutSBitmap)
			return NULL;
	}
	else
	{
		if(pSBitmap->hbm)
			DeleteObject(pSBitmap->hbm);
		pOutSBitmap = pSBitmap;
	}

	// Copy out our bitmap.
	pOutSBitmap->hbm = GetCurrentBitmap(prDim, &pOutSBitmap->cx, &pOutSBitmap->cy);

	return pOutSBitmap;
}

SBitmap *SCartoon::GetCurrentSBitmapMask(SBitmap *pSBitmap, RECT *prDim)
{
	// Returns a pointer to an SBitmap object.
	// (Create one if we need to.)
	SBitmap *pOutSBitmap;

	if(!pSBitmap)
	{
		pOutSBitmap = new SBitmap;
		if(!pOutSBitmap)
			return NULL;
	}
	else
	{
		if(pSBitmap->hbm)
			DeleteObject(pSBitmap->hbm);
		pOutSBitmap = pSBitmap;
	}

	// Copy out our bitmap.
	pOutSBitmap->hbm = GetCurrentBitmapMask(prDim, &pOutSBitmap->cx, &pOutSBitmap->cy);

	return pOutSBitmap;
}

SSprite *SCartoon::GetCurrentSprite(SSprite *pSSprite, RECT *prDim)
{
	SSprite *pOutSSprite;

	if(!BmpFrames)
		return NULL;

	BITMAPFRAME& ThisFrame = BmpFrames[nCurrentDirection * nMaxFrames + nCurrentFrame];

	if(!pSSprite)
	{
		pOutSSprite = new SSprite;
		if(!pOutSSprite)
			return NULL;
	}
	else
		// Bitmaps CANNOT be already loaded in pOutSSprite!!!	
		pOutSSprite = pSSprite;

	pOutSSprite->hbmXOR = ThisFrame.hbmXOR;
	if(pOutSSprite->hbmXOR)
	{
		pOutSSprite->cx = ThisFrame.cx;
		pOutSSprite->cy = ThisFrame.cy;
	}
	pOutSSprite->hbmAND = ThisFrame.hbmAND;

	pOutSSprite->x = x;
	pOutSSprite->y = y;

	pOutSSprite->cxDelta = ThisFrame.cxDelta;
	pOutSSprite->cyDelta = ThisFrame.cyDelta;

	if(prDim)
	{
		if(ThisFrame.hbmAND)
			SetRect(prDim, x, y,
					  x + (int)((float)ThisFrame.cx * fMagnify),
					  y + (int)((float)ThisFrame.cy * fMagnify));
		else
			SetRect(prDim, 0, 0, 0, 0);
	}

	return pOutSSprite;
}

// Calculates new position for object using current frame.
void SCartoon::UpdateNewPosition(int bChangeFrameToo)
{
	if(!BmpFrames)
		return;

	BITMAPFRAME& ThisFrame = BmpFrames[nCurrentDirection * nMaxFrames + nCurrentFrame];

	if(bInMotion)
	{
		// Calculate new X,Y (and magnification) based on current frame.
		if(bUseMagnification)
		{
			// Update magnification factor.  (For simulated 3-D motion.)
			fMagnify *= ThisFrame.fMagnifyFactor;

			// Use it to calculate new (x,y).
			x += (int)((float)ThisFrame.cxDelta * fMagnify);
			y += (int)((float)ThisFrame.cyDelta * fMagnify);
		}
		else
		{
			//  (For 2-D motion)
			x += ThisFrame.cxDelta;
			y += ThisFrame.cyDelta;
		}
	}

	if(bChangeFrameToo)
		nCurrentFrame = ++nCurrentFrame % nMaxFrames;
}

int SCartoon::IsOnEdge(const RECT& rTest)
{
	RECT rSpriteDim;

	// Get the dimensions of our bitmap (including magnification).
	GetCurrentCartoonRect(rSpriteDim);

	// Check to see if sprite touches edge of specified rectangular area,
	//	if so return appropriate edge code.
	if(rSpriteDim.left <= rTest.left)
		return LEFT;
	else if(rSpriteDim.top <= rTest.top)
		return TOP;
	else if(rSpriteDim.right >= rTest.right)
		return RIGHT;
	else if(rSpriteDim.bottom >= rTest.bottom)
		return BOTTOM;
	else
		// Otherwise, sprite is within client area.
		return FALSE;
}


