/*********************************************************
	SCARTOON.H -- 	Header file for a "cartoon" class.
						Manages a series of bitmaps.
*********************************************************/
#if !defined (_SCARTOON_H)
	#define _SCARTOON_H
#include <windows.h>

#include "sbitmap.h"
#include "ssprite.h"

// (Keep your resource names short.)
const int MAXBMPNAMELEN = 20;

typedef struct tagBITMAPFRAME
{
	int nDirection;
	int nFrame;

	int cx;
	int cy;

	int cxDelta;
	int cyDelta;

	float fMagnifyFactor;

	HBITMAP hbmXOR;
	HBITMAP hbmAND;

	char szXORBmpName[MAXBMPNAMELEN];
	char szANDBmpName[MAXBMPNAMELEN];

	int bLoadedOk;
} BITMAPFRAME;

void SetBitmapFrame(BITMAPFRAME& ThisFrame,
						  int nDirection, int nFrame,
						  LPSTR lpszXORBitmapResource,
						  LPSTR lpszANDBitmapResource,
						  int cxDelta, int cyDelta,
						  float fMagnifyFactor = 1.0,
						  int cxNew = 0, int cyNew = 0);

class SCartoon
{
public:
	int x;
	int y;

	float fMagnify;
	int bUseMagnification;  // Allows 3-D-like effects of shrinking and stretching bitmaps.

	int bInMotion;

	int nCurrentDirection;
	int nCurrentFrame;

	int nMaxDirections;
	int nMaxFrames;

	SCartoon(int nNewMaxDirections = 0, int nNewMaxFrames = 0, BITMAPFRAME *pFrames = NULL, int nFrames = 0, HWND hwndFrame = NULL, int bLoadResourcesToo = FALSE);
	// Copy constructor
	SCartoon(const SCartoon& OldCartoon);

	// Assignment operator.
	SCartoon& operator=(const SCartoon& OldCartoon);

	virtual ~SCartoon(void);

	void SetMaxDirections(int nNewMaxDirections) { nMaxDirections = nNewMaxDirections; }
	void SetMaxFrames(int nNewMaxFrames) { nMaxFrames = nNewMaxFrames; }

	// Add entries to BmpFrames Array
	BOOL AddBitmapFramesToArray(BITMAPFRAME *pFrames, int nFrames = 1, int bLoadResourcesToo = FALSE, HWND hwndMyFrame = 0);
	BOOL AddBitmapFrameToArray(int nDirection, int nFrame,
										LPSTR lpszXORBitmapResource,
										LPSTR lpszANDBitmapResource,
										int cxNewDelta, int cyNewDelta,
										float fNewMagnifyFactor = 1.0,
										int cxNew = 0, int cyNew = 0, int bLoadResourcesToo = FALSE, HWND hwndMyFrame = 0);

	void SortBitmapFramesArray(void);

	BOOL LoadResourcesInFramesArray(HWND hwndFrame);
	void CleanupResourcesInFramesArray(void);

	// Outputs a rectangle with our sprite's coordinates,
	// including magnification, if we're using it.
	void GetCurrentCartoonRect(RECT& rDim, int *pcxOriginalSize = NULL, int *pcyOriginalSize = NULL);

	HBITMAP GetCurrentBitmap(RECT *prDim = NULL, int *pcxOriginalSize = NULL, int *pcyOriginalSize = NULL);
	HBITMAP GetCurrentBitmapMask(RECT *prDim = NULL, int *pcxOriginalSize = NULL, int *pcyOriginalSize = NULL);
	SBitmap *GetCurrentSBitmap(SBitmap *pSBitmap = NULL, RECT *prDim = NULL);
	SBitmap *GetCurrentSBitmapMask(SBitmap *pSBitmap = NULL, RECT *prDim = NULL);

	SSprite *GetCurrentSprite(SSprite *pSSprite = NULL, RECT *prDim = NULL);

	// Calculates new position for object using current frame.
	virtual void UpdateNewPosition(int bChangeFrameToo = TRUE);

	void SetMagnifyOptions(BOOL bUseMagnify = TRUE, float fNewMagnify = 1.0) { bUseMagnification = bUseMagnify; fMagnify = fNewMagnify; if(!bUseMagnify) fMagnify = 1.0; }

	// Allows you to 'freeze' a sprite on the screen.
	// You can then watch the sprite 'walking' in-place, frame-by-frame.
	void StopMotion(void) { bInMotion = FALSE; };
	void StartMotion(void) { bInMotion = TRUE; };

	void SetCurrentDirection(int nNewDirection) { nCurrentDirection = nNewDirection; }
	void SetCurrentFrame(int nNewFrame) { nCurrentFrame = nNewFrame; }
	void SetCurrentXY(int xNew, int yNew) { x = xNew; y = yNew; }
	void SetCurrentWindowDim(RECT& rNewDim) { CopyRect(&rNewDim, &rWin); }
	void SetCurrentWindowDim(HWND hwnd) { GetClientRect(hwnd, &rWin); }

	// Override this to provide support for monitoring/responding to
	// on-screen 'collisions.'
	virtual int IsOnEdge(const RECT& rTest);
	virtual int IsOnEdge(HWND hwndFrame) { RECT r;  GetClientRect(hwndFrame, &r); return IsOnEdge(r); }

	// Override to implement a change of direction.
	// Do this is a derived class since we don't know here
	// how many directions are included in the BmpFrames array.
	virtual void ChangeDirection(eEdges EdgeHitCode) {};

protected:
	BITMAPFRAME *BmpFrames;
	int nBmpFramesCount;

	RECT rWin;
	HWND hwndFrame;

	int bInitOk;
};

#endif
