/*********************************************************
	SDIALOG.CPP -- Implementation for "simple" modal
					   dialog boxes.
*********************************************************/
#define MSC_VER
#include "sdialog.h"
#include "resids.h"

// Provides message handling for all our modal dialog boxes.
BOOL FAR PASCAL _export DialogProc(HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam);

SDialog *pActiveDlg = NULL;

SDialog::SDialog(HINSTANCE hInst, HWND hwndMyParent, LPSTR lpszResourceDialogName)
{
	lstrcpy(szResourceName, lpszResourceDialogName);

	hInstDlg = hInst;

   hwndParent = hwndMyParent;

	DlgHandlerList = new LINKEDPTRLIST();
	if(!DlgHandlerList)
	{
		SetError(ALLOC_FAILED, "Can't create list for Dialog Box");
      return;
	}

	nErrCode = 0;
   _fmemset(szErrMsg, 0, 80);
}


SDialog::SDialog(HINSTANCE hInst, HWND hwndMyParent, int nResourceDialogId)
{
	lstrcpy(szResourceName, MAKEINTRESOURCE(nResourceDialogId));

	hInstDlg = hInst;

	hwndParent = hwndMyParent;

	DlgHandlerList = new LINKEDPTRLIST();
	if(!DlgHandlerList)
	{
		SetError(ALLOC_FAILED, "Can't create list for Dialog Box");
      return;
	}
}


SDialog::~SDialog()
{
	if(DlgHandlerList)
	{
		delete DlgHandlerList;
      DlgHandlerList = NULL;
	}
}


int SDialog::Execute(void)
{
	pActiveDlg = this;

#if defined(WIN32)
	// 32-bit version -- no more instance thunks.
	int rc = DialogBox(hInstDlg,
						szResourceName,
						hwndParent,
						(FARPROC)::DialogProc);
#else
	FARPROC lpitDialog = MakeProcInstance((FARPROC)::DialogProc, hInstDlg);

	int rc = DialogBox(hInstDlg,
						szResourceName,
						hwndParent,
						lpitDialog);

	FreeProcInstance(lpitDialog);
#endif
	return rc;
}                        


BOOL FAR PASCAL _export DialogProc(HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam)
{
	WORD wNotify;
	WORD wId;

	pActiveDlg->hwndDlg = hdlg;

	// Message handling.
	switch(msg)
	{
		case WM_INITDIALOG:  // Initialize our window.
			pActiveDlg->SetupWindow();

		case WM_COMMAND:
			wNotify = HIWORD(lParam);
#if defined (WIN32)
			wId = LOWORD(lParam);
#else
			wId = wParam;
#endif
			if(wId == IDOK)
			{
				pActiveDlg->CmOk();
				return TRUE;
			}
			else if(wId == IDCANCEL)
			{
				pActiveDlg->CmCancel();
				return TRUE;
			}
			else
			{
				DLGHANDLERREC *ThisRec;
				// See if our control needs special handling
				for(PTRNODE *Find = (PTRNODE *)pActiveDlg->DlgHandlerList->Head->lpNext;
					 Find != NULL; Find = (PTRNODE *)Find->lpNext)
				{
					ThisRec = (DLGHANDLERREC *)Find->lpData;

					if(ThisRec->nControlId == wId && ThisRec->nNotifyCode == wNotify)
					{
						(*ThisRec->lpfnHandler)(pActiveDlg);
						return TRUE;
					}
				}
				return FALSE;
			}
	}
	wParam;  // Kill warning.
	return FALSE;
}


int SDialog::SetDialogEventHandler(int nCtrlId, int nNotificationCode, HANDLERPROC lpfnHandler)
{
	int rc;
	int bRecFound = FALSE;

	DLGHANDLERREC NewRec;
	DLGHANDLERREC *ThisRec;

	NewRec.nControlId = nCtrlId;
	NewRec.nNotifyCode = nNotificationCode;
   NewRec.lpfnHandler = lpfnHandler;

   // Check to make sure new ID isn't already in list.
	for(PTRNODE *Find = (PTRNODE *)DlgHandlerList->Head->lpNext; Find != NULL; Find=(PTRNODE *)Find->lpNext)
	{
		ThisRec = (DLGHANDLERREC *)Find->lpData;
		if(ThisRec->nControlId == nCtrlId && ThisRec->nNotifyCode == nNotificationCode)
		{
			_fmemcpy(ThisRec, &NewRec, sizeof(NewRec));
         bRecFound = TRUE;
         break;
		}
	}

	if(!bRecFound)
	{
		rc = DlgHandlerList->AddNode((LPSTR)&NewRec, sizeof(NewRec));

		if(!rc)
			return FALSE;
		else
			return TRUE;
	}
   
	return TRUE;
}


void SDialog::ClearDialogEventHandlers(void)
{
	DlgHandlerList->EmptyList();
}


int SDialog::GetCheckInRadioButtonGroup(int nFirstBtnId, int nLastBtnId)
{
	// Returns first--and presumably only--checkbox inside
	// a group of radio buttons.
	for(int i = nFirstBtnId; i <= nLastBtnId; i++)
	{
		if(IsDlgButtonChecked(hwndDlg, i))
			return i;
	}
	return 0;
}


LPSTR SDialog::GetCurrentListboxText(int nLBId)
{
	static char szStringSel[256];

	int nIndex = (int) SendDlgItemMessage(hwndDlg, nLBId, LB_GETCURSEL, 0, 0L);

	if(nIndex >= 0)
      SendDlgItemMessage(hwndDlg, nLBId, LB_GETTEXT, nIndex, (LPARAM)szStringSel);
	else
		_fmemset(szStringSel, 0, 256);

   return szStringSel;
}


LPSTR SDialog::GetCurrentComboboxText(int nCBId, int bUseIndex)
{
	static char szStringSel[256];
   int nIndex;

	if(bUseIndex == -1)
   {
   	// If no index provided, just get current (edited) text.
		GetDlgItemText(hwndDlg, nCBId, szStringSel, 256);
		return szStringSel;
	}
	else
   {
   	nIndex = (int) SendDlgItemMessage(hwndDlg, nCBId, CB_GETCURSEL, 0, 0L);

		if(nIndex >= 0)
      	SendDlgItemMessage(hwndDlg, nCBId, CB_GETLBTEXT, nIndex, (LPARAM)szStringSel);
		else
			_fmemset(szStringSel, 0, 256);
   }
   return szStringSel;
}


// Error routines -- Structured exception handling
//                   will be added in a later version.
int SDialog::GetLastError(LPSTR lpszErrDesc)
{
	if(lpszErrDesc)
		lstrcpy(lpszErrDesc, szErrMsg);
   return nErrCode;
}


void SDialog::ClearError(void)
{
	nErrCode = 0;
   _fmemset(szErrMsg, 0, 80);
}


void SDialog::SetError(int nErr, LPSTR lpszErrMsg)
{
	nErrCode = nErr;
   lstrcpy(szErrMsg, lpszErrMsg);
}


// AboutDlg -- Default class for 'About' dialog box used by SWindows.
SAboutDlg::SAboutDlg(HINSTANCE hInst, HWND hwndMyParent, LPSTR lpszResourceDialogName) :
						   SDialog(hInst, hwndMyParent, lpszResourceDialogName)
{
	SetAboutPrompts("Default About Dialog Box", NULL, NULL, "About");
}


void SAboutDlg::SetAboutPrompts(LPSTR szNewPrompt1, LPSTR szNewPrompt2, LPSTR szNewPrompt3, LPSTR szNewCaption)
{
	if(!szPrompt1)
		memset(szPrompt1, 0, 80);
	else
   	lstrcpy(szPrompt1, szNewPrompt1);

	if(!szPrompt2)
		memset(szPrompt2, 0, 80);
	else
		lstrcpy(szPrompt2, szNewPrompt2);

	if(!szPrompt3)
		memset(szPrompt3, 0, 80);
	else
   	lstrcpy(szPrompt3, szNewPrompt3);

	if(!szCaption)
		memset(szCaption, 0, 80);
	else
   	lstrcpy(szCaption, szNewCaption);
}


void SAboutDlg::SetupWindow(void)
{
	SetText(IDC_PROMPT1, szPrompt1);
	SetText(IDC_PROMPT2, szPrompt2);
	SetText(IDC_PROMPT3, szPrompt3);

   SetWindowText(hwndDlg, szCaption);
}


