/*********************************************************
	SDIALOG.H -- Header file for "simple" modal
				 dialog boxes.
*********************************************************/
#if !defined(_SDIALOG_H)
	#define _SDIALOG_H
#include <windows.h>

#if defined (_MSC_VER)
	#include "msc_fix.h"
#else
	#include <mem.h>
#endif

#include "slist.h"

// Override this to provide custom event handling.
// BOOL FAR PASCAL _export DialogProc(HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam);

class SDialog
{
public:
	SDialog(HINSTANCE hInst, HWND hwndMyParent, LPSTR lpszResourceDialogName);
	SDialog(HINSTANCE hInst, HWND hwndMyParent, int nResourceDialogId);

	~SDialog();

	int Execute(void);
	int ShowDialog(void) { return Execute(); };

	// REM BOOL FAR PASCAL _export DialogProc(HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam);

	virtual void SetupWindow(void){};

	virtual void CmOk(void) { CloseDialog(TRUE); };
	virtual void CmCancel(void) { CloseDialog(FALSE); };

	virtual void CleanupWindow(void){};

	void CloseDialog(int nReturnCode) { EndDialog(hwndDlg, nReturnCode); };
	void CloseWindow(int nReturnCode) { EndDialog(hwndDlg, nReturnCode); };

	// Work with controls -- register handlers
	int SetDialogEventHandler(int nCtrlId, int nNotificationId, HANDLERPROC lpfnHandler);
	void ClearDialogEventHandlers(void);

	// Control helper functions.
	void SetText(int nControlId, LPSTR lpszNewText) { SetDlgItemText(hwndDlg, nControlId, lpszNewText); };
	void GetText(int nControlId, LPSTR lpszText, int nMaxChars){ GetDlgItemText(hwndDlg, nControlId, lpszText, nMaxChars); };

	void DisableControl(int nControlId) { EnableWindow(GetDlgItem(hwndDlg, nControlId), TRUE); };
	void EnableControl(int nControlId) { EnableWindow(GetDlgItem(hwndDlg, nControlId), FALSE); };

	void SetCheckBox(int nCheckBoxId, int bIsChecked) { CheckDlgButton(hwndDlg, nCheckBoxId, TRUE); };
	int GetCheckBox(int nCheckBoxId) { return IsDlgButtonChecked(hwndDlg, nCheckBoxId); };

	void CheckRadioButtonInGroup(int nFirstBtnId, int nLastBtnId, int RadioBtnIdToCheck){
					CheckRadioButton(hwndDlg, nFirstBtnId, nLastBtnId, RadioBtnIdToCheck);
					};

	int IsRadioButtonChecked(int nRadioBoxId) { return IsDlgButtonChecked(hwndDlg, nRadioBoxId); };
	int GetCheckInRadioButtonGroup(int nFirstBtnId, int nLastBtnId);

	void ClearListbox(int nLBId) { SendDlgItemMessage(hwndDlg, nLBId, LB_RESETCONTENT, 0, 0L); };
	int AddListboxEntry(int nLBId, LPSTR lpszEntry) { return (int)SendDlgItemMessage(hwndDlg, nLBId, LB_ADDSTRING, 0, (LPARAM)lpszEntry); };
	int InsertListboxEntry(int nLBId, LPSTR lpszEntry, int nPosition){ return (int)SendDlgItemMessage(hwndDlg, nLBId, LB_INSERTSTRING, nPosition, (LPARAM)lpszEntry); };

	void SetCurrentListboxSelection(int nLBId, int nIndex) { SendDlgItemMessage(hwndDlg, nLBId, LB_SETCURSEL, nIndex, 0L); };
	int GetCurrentListboxSelection(int nLBId) { return (int) SendDlgItemMessage(hwndDlg, nLBId, LB_GETCURSEL, 0, 0L); };
	LPSTR GetCurrentListboxText(int nLBId);

	int SetListboxHorizontalExtent(int nLBId, int cxWidthPixels) { return (int)SendDlgItemMessage(hwndDlg, nLBId, LB_SETHORIZONTALEXTENT, cxWidthPixels, 0L); };

	// For multiple selection listboxes.
	int IsListboxIndexSelected(int nLBId, int nIndex) { return (int)SendDlgItemMessage(hwndDlg, nLBId, LB_GETSEL, nIndex, 0L); };

	// Combobox methods.
	void ClearCombobox(int nCBId) { SendDlgItemMessage(hwndDlg, nCBId, CB_RESETCONTENT, 0, 0L); };
	int AddComboboxEntry(int nCBId, LPSTR lpszCBEntry) { return (int)SendDlgItemMessage(hwndDlg, nCBId, CB_ADDSTRING, 0, (LPARAM)lpszCBEntry); };
	int InsertComboboxEntry(int nCBId, int nIndex, LPSTR lpszCBEntry) { return (int)SendDlgItemMessage(hwndDlg, nCBId, CB_INSERTSTRING, nIndex, (LPARAM)lpszCBEntry); };

	void SetCurrentComboboxSelection(int nCBId, int nIndex) { SendDlgItemMessage(hwndDlg, nCBId, CB_SETCURSEL, nIndex, 0L); };
	int GetCurrentComboboxSelection(int nCBId) { return (int)SendDlgItemMessage(hwndDlg, nCBId, CB_GETCURSEL, 0, 0L); };
	LPSTR GetCurrentComboboxText(int nCBId, int nIndex = -1);

	// For multiple selection comboboxes.
	int IsComboboxIndexSelected(int nLBId, int bUseIndex = FALSE);

	// Error routines -- Structured exception handling
	//                   will be added in a later version.
	int GetLastError(LPSTR lpszErrDesc = NULL);
	void ClearError(void);
	void SetError(int nErr, LPSTR lpszErrMsg);

	LINKEDPTRLIST *DlgHandlerList;
	HWND hwndDlg;

protected:
	char szResourceName[40];

	HWND hwndParent;
	HINSTANCE hInstDlg;

private:
	int nErrCode;
	char szErrMsg[80];
};


// An 'About' Dialog Class.
class SAboutDlg : public SDialog
{
public:
	SAboutDlg(HINSTANCE hInst, HWND hwndMyParent, LPSTR lpszResourceDialogName = "ABOUT_DLG");
	~SAboutDlg(){};

	void SetupWindow(void);

	void SetAboutPrompts(LPSTR lpszNewPrompt1 = "About this program.", LPSTR lpszNewPrompt2 = NULL,
								LPSTR lpszNewPrompt3 = NULL, LPSTR lpszNewCaption = "About");

private:
	char szPrompt1[80];
	char szPrompt2[80];
	char szPrompt3[80];
	char szCaption[80];
};

#endif
