/*********************************************************
	SDRAW.CPP -- Implementation for "simple" Windows DC / GDI
					 encapsulation.

				Includes support for:

				-- Basic GDI drawing--brush/pen/font/ROP2 mode
					manipulation.
				-- Off-screen buffering.
				-- Draws other SWindows objects--SBitmap's,
					SSprite's, SCartoon's.
*********************************************************/
#include "sdrawcor.h"
#include "sdraw.h"
#include "msc_fix.h"
#include <stdlib.h>

// Return a valid "random" color...
DWORD GetRandomColor(void)
{
	DWORD dwColor;

	int n = random(20);

	switch(n)
	{
		case 0: 	dwColor = White;
					break;
		case 1:  dwColor = Gray;
					break;
		case 2:  dwColor = LtGray;
					break;
		case 3:  dwColor = DarkRed;
					break;
		case 4: 	dwColor = Red;
					break;
		case 5: 	dwColor = DarkYellow;
					break;
		case 6: 	dwColor = Yellow;
					break;
		case 7: 	dwColor = LtYellow;
					break;
		case 8: 	dwColor = DarkGreen;
					break;
		case 9: 	dwColor = Green;
					break;
		case 10: dwColor = DarkCyan;
					break;
		case 11: dwColor = Cyan;
					break;
		case 12: dwColor = LtCyan;
					break;
		case 13: dwColor = DarkBlue;
					break;
		case 14: dwColor = Blue;
					break;
		case 15: dwColor = Purple;
					break;
		case 16: dwColor = Magenta;
					break;
		case 17: dwColor = Brown;
					break;
		case 18: dwColor = Orange;
					break;
		default: dwColor = Black;
	}
	return dwColor;
}


// Constructor
SDraw::SDraw(HWND hwnd, SDrawCore *pCore)
{
	bGraphicsInitOk = FALSE;

	hwndFrame = hwnd;
	bCoreWasCreatedByUs = FALSE;

	nDefaultWidth = 1;

	if(!pCore)
	{
		pCoreModule = new SDrawCore(hwnd);
		if(!pCoreModule)
			MessageBox(NULL, "Can't create SDrawCore Object!", "Can't Draw", MB_ICONSTOP);
		else
		{
			bCoreWasCreatedByUs = TRUE;
			bGraphicsInitOk = TRUE;
		}
	}
	else
		bGraphicsInitOk = TRUE;
}

// Destructor
SDraw::~SDraw()
{
	if(bCoreWasCreatedByUs)
		delete pCoreModule;
}

void SDraw::StartDrawing(int bDuringPaintEvent)
{
	if(!bGraphicsInitOk)
		return;
	if(pCoreModule->bBufferScreenUpdates == FALSE)
	{
		if(bDuringPaintEvent)
			hdcWindow = BeginPaint(hwndFrame, &ps);
		else
		{
			if(!hwndFrame)
				return;
			hdcWindow = GetDC(hwndFrame);
		}
		pCoreModule->hdcActive = hdcWindow;
	}
	pCoreModule->hdcDefault = hdcWindow;
}

void SDraw::StopDrawing(int bDuringPaintEvent)
{
	if(!bGraphicsInitOk)
		return;
	if(pCoreModule->bBufferScreenUpdates == FALSE)
	{
		if(bDuringPaintEvent)
			EndPaint(hwndFrame, &ps);
		else
			ReleaseDC(hwndFrame, hdcWindow);
	}
}

// GDI API encapsulation.
void SDraw::SetPixel(int x, int y, DWORD Color)
{
	if(!bGraphicsInitOk)
		return;
	if(Color != Transparent)
		::SetPixel(pCoreModule->hdcActive, x, y, Color);
}

void SDraw::SetPixel(POINT pt, DWORD Color)
{
	if(!bGraphicsInitOk)
		return;
	if(Color != Transparent)
		::SetPixel(pCoreModule->hdcActive, pt.x, pt.y, Color);
}

void SDraw::GetPixel(int x, int y, DWORD& Color)
{
	if(!bGraphicsInitOk)
		return;
	Color = ::SetPixel(pCoreModule->hdcActive, x, y, Color);
}

void SDraw::GetPixel(POINT pt, DWORD& Color)
{
	if(!bGraphicsInitOk)
		return;
	Color = ::GetPixel(pCoreModule->hdcActive, pt.x, pt.y);
}

DWORD SDraw::GetPixel(int x, int y)
{
	if(!bGraphicsInitOk)
		 return RGB(0,0,0);
	return ::GetPixel(pCoreModule->hdcActive, x, y);
}


//	BOOL SelectFont(HDC hdc, char *lpszFaceName, int nPoints, int bIsBold);
void SDraw::Line(int x1, int y1, int x2, int y2, DWORD Color, int nWidth)
{
	if(!bGraphicsInitOk)
		return;

	SelectPen(pCoreModule->hdcActive,Color, nWidth);

#if defined (WIN32)
	::MoveToEx(pCoreModule->hdcActive, x1, y1, NULL);
#else
	::MoveTo(pCoreModule->hdcActive, x1, y1);
#endif
	::LineTo(pCoreModule->hdcActive, x2, y2);
}

void SDraw::Line(POINT pt1, POINT pt2, DWORD Color, int nWidth)
{
	if(!bGraphicsInitOk)
		return;

	SelectPen(pCoreModule->hdcActive,Color, nWidth);

#if defined (WIN32)
	::MoveToEx(pCoreModule->hdcActive, pt1.x, pt1.y, NULL);
#else
	::MoveTo(pCoreModule->hdcActive, pt1.x, pt1.y);
#endif
	::LineTo(pCoreModule->hdcActive, pt2.x, pt2.y);
}

void SDraw::MoveTo(int x, int y)
{
	if(!bGraphicsInitOk)
		return;

#if defined (WIN32)
	::MoveToEx(pCoreModule->hdcActive, x, y, NULL);
#else
	::MoveTo(pCoreModule->hdcActive, x, y);
#endif
}

void SDraw::MoveTo(POINT pt)
{
	if(!bGraphicsInitOk)
		return;

#if defined (WIN32)
	::MoveToEx(pCoreModule->hdcActive, pt.x, pt.y, NULL);
#else
	::MoveTo(pCoreModule->hdcActive, pt.x, pt.y);
#endif
}


void SDraw::LineTo(int x, int y, DWORD Color, int nWidth)
{
	if(!bGraphicsInitOk)
		return;

	SelectPen(pCoreModule->hdcActive, Color, nWidth);

	::LineTo(pCoreModule->hdcActive, x, y);
}

void SDraw::LineTo(POINT pt, DWORD Color, int nWidth)
{
	if(!bGraphicsInitOk)
		return;

	SelectPen(pCoreModule->hdcActive, Color, nWidth);

	::LineTo(pCoreModule->hdcActive, pt.x, pt.y);
}

void SDraw::Rectangle(int x1, int y1, int x2, int y2, DWORD Color, int bBorderOnly)
{
	if(!bGraphicsInitOk)
		return;

	SelectPen(pCoreModule->hdcActive, Color, nDefaultWidth);

	if(bBorderOnly)
		SelectBrush(pCoreModule->hdcActive, Transparent);
	else
		SelectBrush(pCoreModule->hdcActive, Color);

	::Rectangle(pCoreModule->hdcActive, x1, y1, x2, y2);
}


void SDraw::Rectangle(POINT pt1, POINT pt2, DWORD Color, int bBorderOnly)
{
	if(!bGraphicsInitOk)
		return;

	SelectPen(pCoreModule->hdcActive, Color, nDefaultWidth);

	if(bBorderOnly)
		SelectBrush(pCoreModule->hdcActive, Transparent);
	else
		SelectBrush(pCoreModule->hdcActive, Color);

	::Rectangle(pCoreModule->hdcActive, pt1.x, pt1.y, pt2.x, pt2.y);
}

void SDraw::Ellipse(int x1, int y1, int x2, int y2, DWORD Color, int bBorderOnly)
{
	if(!bGraphicsInitOk)
		return;

	SelectPen(pCoreModule->hdcActive, Color, nDefaultWidth);

	if(bBorderOnly)
		SelectBrush(pCoreModule->hdcActive, Transparent);
	else
		SelectBrush(pCoreModule->hdcActive, Color);

	::Ellipse(pCoreModule->hdcActive, x1, y1, x2, y2);
}

void SDraw::Ellipse(POINT pt1, POINT pt2, DWORD Color, int bBorderOnly)
{
	if(!bGraphicsInitOk)
		return;

	SelectPen(pCoreModule->hdcActive, Color, nDefaultWidth);

	if(bBorderOnly)
		SelectBrush(pCoreModule->hdcActive, Transparent);
	else
		SelectBrush(pCoreModule->hdcActive, Color);

	::Ellipse(pCoreModule->hdcActive, pt1.x, pt1.y, pt2.x, pt2.y);
}

void SDraw::SetLineWidth(int nWidth)
{
	nDefaultWidth = nWidth;
}

void SDraw::SetROP2(DWORD dwROP2)
{
	if(!bGraphicsInitOk)
		return;

	::SetROP2(pCoreModule->hdcActive, (UINT)dwROP2);
}

// Font and text manipulation
void SDraw::SetFont(LPSTR lpszFaceName, int nPoints, int bIsBold)
{
	if(!bGraphicsInitOk)
		return;

	SelectFont(pCoreModule->hdcActive, lpszFaceName, nPoints, bIsBold);
}

void SDraw::SetTextForeColor(DWORD FColor)
{
	if(!bGraphicsInitOk)
		return;

	::SetTextColor(pCoreModule->hdcActive, FColor);
}

void SDraw::SetTextBackColor(DWORD BColor)
{
	if(!bGraphicsInitOk)
		return;

	::SetBkColor(pCoreModule->hdcActive, BColor);
}

void SDraw::DisplayText(int x, int y, LPSTR lpszText, DWORD Color)
{
	if(!bGraphicsInitOk)
		return;

	if(Color != (DWORD)-1)
		::SetTextColor(pCoreModule->hdcActive, Color);

	::TextOut(pCoreModule->hdcActive, x, y, lpszText, lstrlen(lpszText));
}

void SDraw::DisplayText(POINT pt, LPSTR lpszText, DWORD Color)
{
	if(!bGraphicsInitOk)
		return;

	if(Color != (DWORD)-1)
		::SetTextColor(pCoreModule->hdcActive, Color);

	::TextOut(pCoreModule->hdcActive, pt.x, pt.y, lpszText, lstrlen(lpszText));
}

void SDraw::DisplayText(RECT& r, LPSTR lpszText, DWORD Color)
{
	if(!bGraphicsInitOk)
		return;

	if(Color != (DWORD)-1)
		::SetTextColor(pCoreModule->hdcActive, Color);

	::DrawText(pCoreModule->hdcActive, lpszText, lstrlen(lpszText), &r, DT_LEFT | DT_WORDBREAK);
}

void SDraw::TextOut(int x, int y, LPSTR lpszText, int nLength)
{
	if(!bGraphicsInitOk)
		return;

	::TextOut(pCoreModule->hdcActive, x, y, lpszText, nLength);
}

void SDraw::DrawText(LPSTR lpszText, RECT& rClip, int nOptions)
{
	if(!bGraphicsInitOk)
		return;

	::DrawText(pCoreModule->hdcActive, lpszText, lstrlen(lpszText), &rClip, nOptions);
}

void SDraw::ExtTextOut(int x, int y, UINT uFlags, RECT& rClip, LPSTR lpszText, int nLength)
{
	if(!bGraphicsInitOk)
		return;

	::ExtTextOut(pCoreModule->hdcActive, x, y, uFlags, &rClip, lpszText, nLength, NULL);
}

// Image/bitmap routines.
void SDraw::DrawBitmap(int x, int y, SBitmap& image, DWORD dwROP)
{
	if(!bGraphicsInitOk)
		return;

	HBITMAP hbmOld = SelectObject(pCoreModule->hdcMemory, image.hbm);

	BitBlt(pCoreModule->hdcActive, x, y, image.cx, image.cy,
			 pCoreModule->hdcMemory, 0, 0, dwROP);

	SelectObject(pCoreModule->hdcMemory, hbmOld);
}

void SDraw::DrawBitmap(POINT pt, SBitmap& image, DWORD dwROP)
{
	if(!bGraphicsInitOk)
		return;

	HBITMAP hbmOld = SelectObject(pCoreModule->hdcMemory, image.hbm);

	BitBlt(pCoreModule->hdcActive, pt.x, pt.y, image.cx, image.cy,
			 pCoreModule->hdcMemory, 0, 0, dwROP);

	SelectObject(pCoreModule->hdcMemory, hbmOld);
}


void SDraw::StretchBitmap(int x, int y, int nNewWidth, int nNewHeight, SBitmap& image, DWORD dwROP)
{
	if(!bGraphicsInitOk)
		return;

	HBITMAP hbmOld = SelectObject(pCoreModule->hdcMemory, image.hbm);

	StretchBlt(pCoreModule->hdcActive, x, y, nNewWidth, nNewHeight,
			 pCoreModule->hdcMemory, 0, 0, image.cx, image.cy, dwROP);

	SelectObject(pCoreModule->hdcMemory, hbmOld);
}

void SDraw::StretchBitmap(POINT pt1, POINT pt2, SBitmap& image, DWORD dwROP)
{
	if(!bGraphicsInitOk)
		return;

	HBITMAP hbmOld = SelectObject(pCoreModule->hdcMemory, image.hbm);

	StretchBlt(pCoreModule->hdcActive, pt1.x, pt1.y, abs(pt2.x- pt1.x), abs(pt2.y - pt2.y),
			 pCoreModule->hdcMemory, 0, 0, image.cx, image.cy, dwROP);

	SelectObject(pCoreModule->hdcMemory, hbmOld);
}

// Sprite routines.
void SDraw::DrawSprite(SSprite& sprite)
{
	if(!bGraphicsInitOk)
		return;

	HBITMAP hbmOld = SelectObject(pCoreModule->hdcMemory, sprite.hbmAND);

	BitBlt(pCoreModule->hdcActive, sprite.x, sprite.y, sprite.cx, sprite.cy,
			 pCoreModule->hdcMemory, 0, 0, SRCAND);

	SelectObject(pCoreModule->hdcMemory, sprite.hbmXOR);

	BitBlt(pCoreModule->hdcActive, sprite.x, sprite.y, sprite.cx, sprite.cy,
			 pCoreModule->hdcMemory, 0, 0, SRCINVERT);

	SelectObject(pCoreModule->hdcMemory, hbmOld);
}

void SDraw::DrawSprite(int x, int y, SSprite& sprite)
{
	if(!bGraphicsInitOk)
		return;

	HBITMAP hbmOld = SelectObject(pCoreModule->hdcMemory, sprite.hbmAND);

	BitBlt(pCoreModule->hdcActive, x, y, sprite.cx, sprite.cy,
			 pCoreModule->hdcMemory, 0, 0, SRCAND);

	SelectObject(pCoreModule->hdcMemory, sprite.hbmXOR);

	BitBlt(pCoreModule->hdcActive, x, y, sprite.cx, sprite.cy,
			 pCoreModule->hdcMemory, 0, 0, SRCINVERT);

	SelectObject(pCoreModule->hdcMemory, hbmOld);
}

void SDraw::DrawSprite(POINT pt, SSprite& sprite)
{
	if(!bGraphicsInitOk)
		return;

	HBITMAP hbmOld = SelectObject(pCoreModule->hdcMemory, sprite.hbmAND);

	BitBlt(pCoreModule->hdcActive, pt.x, pt.y, sprite.cx, sprite.cy,
			 pCoreModule->hdcMemory, 0, 0, SRCAND);

	SelectObject(pCoreModule->hdcMemory, sprite.hbmXOR);

	BitBlt(pCoreModule->hdcActive, pt.x, pt.y, sprite.cx, sprite.cy,
			 pCoreModule->hdcMemory, 0, 0, SRCINVERT);

	SelectObject(pCoreModule->hdcMemory, hbmOld);
}

// Special case, allows you to draw a sprite from
// a Mem. DC and two rectangular areas to copy out.
// (Can be used for better performance with SSprite objects.)
void SDraw::DrawSprite(POINT pt, SSprite& sprite, RECT& rXORBmp, RECT& rANDBmp)
{
	if(!bGraphicsInitOk)
		return;

	int cxUse = rXORBmp.right - rXORBmp.left;
	int cyUse = rXORBmp.bottom - rXORBmp.top;

	SelectObject(pCoreModule->hdcMemory, sprite.hbmAND);

	BitBlt(pCoreModule->hdcActive, pt.x, pt.y, cxUse, cyUse,
			 pCoreModule->hdcMemory, rANDBmp.left, rANDBmp.top, SRCAND);

	SelectObject(pCoreModule->hdcMemory, sprite.hbmXOR);

	BitBlt(pCoreModule->hdcActive, pt.x, pt.y, cxUse, cyUse,
			 pCoreModule->hdcMemory, rXORBmp.left, rXORBmp.top, SRCINVERT);
}

void SDraw::StretchSprite(int nNewWidth, int nNewHeight, SSprite& sprite)
{
	if(!bGraphicsInitOk)
		return;

	HBITMAP hbmOld = SelectObject(pCoreModule->hdcMemory, sprite.hbmAND);

	StretchBlt(pCoreModule->hdcActive, sprite.x, sprite.y, nNewWidth, nNewHeight,
			 pCoreModule->hdcMemory, 0, 0, sprite.cx, sprite.cy, SRCAND);

	SelectObject(pCoreModule->hdcMemory, sprite.hbmXOR);

	StretchBlt(pCoreModule->hdcActive, sprite.x, sprite.y, nNewWidth, nNewHeight,
			 pCoreModule->hdcMemory, 0, 0, sprite.cx, sprite.cy, SRCINVERT);

	SelectObject(pCoreModule->hdcMemory, hbmOld);
}

void SDraw::StretchSprite(int x, int y, int nNewWidth, int nNewHeight, SSprite& sprite)
{
	if(!bGraphicsInitOk)
		return;

	HBITMAP hbmOld = SelectObject(pCoreModule->hdcMemory, sprite.hbmAND);

	StretchBlt(pCoreModule->hdcActive, x, y, nNewWidth, nNewHeight,
			 pCoreModule->hdcMemory, 0, 0, sprite.cx, sprite.cy, SRCAND);

	SelectObject(pCoreModule->hdcMemory, sprite.hbmXOR);

	StretchBlt(pCoreModule->hdcActive, x, y, nNewWidth, nNewHeight,
			 pCoreModule->hdcMemory, 0, 0, sprite.cx, sprite.cy, SRCINVERT);

	SelectObject(pCoreModule->hdcMemory, hbmOld);
}

void SDraw::StretchSprite(POINT pt1, POINT pt2, SSprite& sprite)
{
	if(!bGraphicsInitOk)
		return;

	HBITMAP hbmOld = SelectObject(pCoreModule->hdcMemory, sprite.hbmAND);

	int cxUse = abs(pt2.x - pt1.x);
	int cyUse = abs(pt2.y - pt1.y);

	StretchBlt(pCoreModule->hdcActive, pt1.x, pt1.y, cxUse, cyUse,
			 pCoreModule->hdcMemory, 0, 0, sprite.cx, sprite.cy, SRCAND);

	SelectObject(pCoreModule->hdcMemory, sprite.hbmXOR);

	StretchBlt(pCoreModule->hdcActive, pt1.x, pt1.y, cxUse, cyUse,
			 pCoreModule->hdcMemory, 0, 0, sprite.cx, sprite.cy, SRCINVERT);

	SelectObject(pCoreModule->hdcMemory, hbmOld);
}


void SDraw::DrawCartoon(int x, int y, SCartoon& cartoon)
{
	// Draw a cartoon object.
	// (If we're using 'magnification,' stretching and shrinking a bitmap
	// to simulate a 3-D effect, use StretchBlt()--otherwise, use BitBlt().)
	if(!bGraphicsInitOk)
		return;

	RECT rXOR;  // We're assuming XOR mask is the same size as the AND mask.
	int bmOrigWidth;
	int bmOrigHeight;

	HBITMAP hbmXOR = cartoon.GetCurrentBitmap(&rXOR, &bmOrigWidth, &bmOrigHeight);
	HBITMAP hbmAND = cartoon.GetCurrentBitmapMask(NULL);

	HBITMAP hbmOld = SelectObject(pCoreModule->hdcMemory, hbmAND);

	if(!cartoon.bUseMagnification)
	{
		// The simple case--use BitBlt()
		BitBlt(pCoreModule->hdcActive, x, y, bmOrigWidth, bmOrigHeight,
			 pCoreModule->hdcMemory, 0, 0, SRCAND);

		SelectObject(pCoreModule->hdcMemory, hbmXOR);

		BitBlt(pCoreModule->hdcActive, x, y, bmOrigWidth, bmOrigHeight,
			 pCoreModule->hdcMemory, 0, 0, SRCINVERT);
	}
	else
	{
		// Use StretchBlt() instead.
		int cxUse = abs(rXOR.right - rXOR.left);
		int cyUse = abs(rXOR.bottom - rXOR.top);

		StretchBlt(pCoreModule->hdcActive, x, y, cxUse, cyUse,
				 pCoreModule->hdcMemory, 0, 0, bmOrigWidth, bmOrigHeight, SRCAND);

		SelectObject(pCoreModule->hdcMemory, hbmXOR);

		StretchBlt(pCoreModule->hdcActive, x, y, cxUse, cyUse,
				 pCoreModule->hdcMemory, 0, 0, bmOrigWidth, bmOrigHeight, SRCINVERT);
	}
	SelectObject(pCoreModule->hdcMemory, hbmOld);
}


void SDraw::StartScreenBuffering(void)
{
	if(!bGraphicsInitOk)
		return;

	pCoreModule->StartScreenBuffering();
}

void SDraw::StopScreenBuffering(void)
{
	if(!bGraphicsInitOk)
		return;

	pCoreModule->StopScreenBuffering();
}

void SDraw::CopyScreenToBuffer(int cxCopy, int cyCopy)
{
	if(!bGraphicsInitOk)
		return;

	pCoreModule->CopyScreenToBuffer(hwndFrame, cxCopy, cyCopy);

}

void SDraw::CopyBufferToScreen(int cxCopy, int cyCopy)
{
	if(!bGraphicsInitOk)
		return;

	pCoreModule->CopyBufferToScreen(hwndFrame, cxCopy, cyCopy);
}


#if defined (INCL_DEBUG)
void SDraw::ReportToolUsage(HWND hwnd)
{
	if(!bGraphicsInitOk)
		return;

	pCoreModule->ReportToolUsage(hwnd);  // Forward this request.
}
#endif

