/*********************************************************
	SDRAWCOR.H -- Header file for "simple" Windows DC / GDI
					  encapsulation -- Core routines to manage
					  off-screen buffering and GDI tool
					  management.

*********************************************************/
#if !defined (_SDRAWCOR_H)
	#define _SDRAWCOR_H
#include <windows.h>
#include <string.h>

#include "colors.h"
#include "slist.h"

#define INCL_DEBUG

#define HGDITOOL HANDLE

enum eToolType { None, Brush, Pen, Font };

// A few helper classes.
class TOOLDESC
{
public:
	eToolType TypeCode;
	DWORD rgbColor;
	int nPenWidth;
	HGDITOOL hTool;
	TOOLDESC(eToolType eType = None, DWORD rgbMyColor = Black, int nWidth = 1) {
					TypeCode = eType;
					rgbColor = rgbMyColor;
					nPenWidth = nWidth;
					hTool = NULL; }
	TOOLDESC& operator=(const TOOLDESC& rOld) { TypeCode = rOld.TypeCode;
												 rgbColor = rOld.rgbColor;
												 nPenWidth = rOld.nPenWidth;
												 hTool = rOld.hTool;
												 return *this;	 }
	BOOL operator==(const TOOLDESC& rTest) { if(TypeCode == rTest.TypeCode && rgbColor == rTest.rgbColor)
															return TRUE;
														else
															return FALSE; }
	virtual int CreateIt(void);
	virtual void DeleteIt(void);
};

class FONTDESC : public TOOLDESC
{
public:
	BOOL bIsBold;
	char szFaceName[50];
	int nPoints;

	FONTDESC(char *lpszFace = "", int nPts = 10, int bBold = FALSE, int rgbMyColor = Black) : TOOLDESC(Font)
					{ TypeCode = Font; strcpy(szFaceName, lpszFace); nPoints = nPts; bIsBold = bBold; rgbColor = rgbMyColor; }
	FONTDESC& operator=(const FONTDESC& rOld) { TypeCode = rOld.TypeCode;
												 rgbColor = rOld.rgbColor;
												 nPenWidth = rOld.nPenWidth;  // Shouldn't contain useful data.
												 strcpy(szFaceName, rOld.szFaceName);
												 nPoints = rOld.nPoints;
												 bIsBold = rOld.bIsBold;
												 return *this; }
	BOOL operator==(const FONTDESC& rTest) { if(TypeCode == rTest.TypeCode && nPoints == rTest.nPoints
															&& strcmpi(szFaceName, rTest.szFaceName) == 0 && bIsBold == rTest.bIsBold)
																return TRUE;
															else
																return FALSE; }
	virtual int CreateIt(void);
};


class SDrawCore
{
public:
	// Constructor and destructor
	//SDrawCore(SWindows& swindows);
	SDrawCore(HWND hwnd);
	//SDrawCore(HDC hdcScreen);
	~SDrawCore();

	TOOLDESC *DoesGDIToolExist(int Type, DWORD rgbColor, int nPenWidth);

	FONTDESC *DoesFontExist(char *lpszFaceName, int nPoints, int bIsBold);

	TOOLDESC *AddGDITool(int Type, DWORD rgbColor, int nPenWidth);


	FONTDESC *AddFont(char *lpszFaceName, int nPoints, int bIsBold);

	TOOLDESC *GetTool(eToolType Type, DWORD rgbColor, int nPenWidth = 1);

	FONTDESC *GetFont(LPSTR lpszFaceName, int nPoints, int bIsBold);

	// Select Tools
	BOOL SelectBrush(HDC hdc, DWORD rgbColor);
	BOOL SelectPen(HDC hdc, DWORD rgbColor, int nPenWidth);
	BOOL SelectFont(HDC hdc, char *lpszFaceName, int nPoints, int bIsBold);

	void RestoreOldBrush(HDC hdc);
	void RestoreOldPen(HDC hdc);
	void RestoreOldFont(HDC hdc);

	void SetDrawingSurface(HDC hdcNew = 0);

	void CopyScreenToBuffer(HWND hwnd, int cxCopy = 0, int cyCopy = 0);
	void CopyBufferToScreen(HWND hwnd, int cxCopy = 0, int cyCopy = 0);

	void StartScreenBuffering(void);
	void StopScreenBuffering(void);  //I.e. write directly to screen!

	void ClearOffscreenBuffer(void);

	BOOL bBufferScreenUpdates;

	HDC hdcMemory; // For rendering bitmaps back and forth between DC's.

	HDC hdcDefault; // Default screen DC

	HDC hdcActive;  // Can be hdcMemory -OR- hdcDefault -- Allows us to
						 // buffer screen updates by drawing to offscreen DC.

#if defined (INCL_DEBUG)
	// For Debugging.
	void ReportToolUsage(HWND hwnd);
#endif

private:
	HDC hdcOffScreen;
	HBITMAP hbmDefaultOffscreenBitmap;
	HBITMAP hbmDefaultMemoryBitmap;
	HBITMAP hbmCurrentOffscreen;

	HPALETTE hpalOffscreen;
	int cxMaxScreen;
	int cyMaxScreen;
	int nBitsPerPixelOffscreen;  // Color depth.

	LINKEDPTRLIST *GDIToolsList;

	HPEN hpenOld;
	HBRUSH hbrOld;
	HFONT hfontOld;

};
#endif
