/***************************************************************
	SIM.CPP -- Implementation of custom sprites and a simulation
				  control object.
***************************************************************/
#include "msc_fix.h"
#include "sim.h"
#include <memory.h>

#pragma argsused
SimMushroom::SimMushroom(HWND hwnd, LPSTR lpszXORBmpName, LPSTR lpszANDBmpName, int xPos, int yPos) :
	SSprite(hwnd, lpszXORBmpName, lpszANDBmpName, xPos, yPos)
{
}

void SimMushroom::InitPos(int xNew, int yNew)
{
	// Sets new X, Y and calculates cxDelta.
	// Assumes bitmaps are loaded!
	x = xNew;
	y = yNew;

	// Mushrooms move left and right (slowly)
	cxDelta = 3 + random(6);
	if(cxDelta == 0)
		cxDelta = 1;
	cyDelta = 0;
}

SimCentipede::SimCentipede(HWND hwnd, LPSTR lpszXORBmpName, LPSTR lpszANDBmpName)
{
	bInitOk = FALSE;

	// NULL out all members.  Call init. method if arg's provided.
	nSegmentCount = 0;
	nSegmentsInCentipede = 0;

	hdcActive = NULL;

	bDownScreen = TRUE;
	nWiggleCode = 0;

	memset(CentiSegments, 0, sizeof(CENTISEGMENTINFO) * MAXCENTISEGMENTS);

	memset(&rWin, 0, sizeof(RECT));

	if(hwnd && lpszXORBmpName && lpszANDBmpName)
		bInitOk = Init(hwnd, lpszXORBmpName, lpszANDBmpName);
}


int SimCentipede::Init(HWND hwnd, LPSTR lpszXORBmpName, LPSTR lpszANDBmpName)
{
	if(!hwnd || !lpszXORBmpName || !lpszANDBmpName)
		return FALSE;

	// First, load panel centipede cels.
	int rc = LoadBitmaps(hwnd, lpszXORBmpName, lpszANDBmpName);
	if(!rc)
		return FALSE;

	SaveWindowSize(hwnd);

	return TRUE;
}


void SimCentipede::SaveWindowSize(HWND hwnd)
{
	GetClientRect(hwnd, &rWin);
}


// Destructor
SimCentipede::~SimCentipede(void)
{
}


void SimCentipede::InitNewCentipede(int cxNewDelta, int nSegments, int bMoveDownScreen)
{
	int xFirstSegment;
	int yFirstSegment;

	cxDelta = cxNewDelta;

	// Generates a new 'pede...
	nSegmentCount = 0;
	nSegmentsInCentipede = nSegments;

	int nNewWiggleCode = 5;

	for(int i = 0; i < nSegmentsInCentipede; i++)
	{
		CentiSegments[i].nWiggleCode = nNewWiggleCode;
		nNewWiggleCode = nNewWiggleCode == 5 ? 1 : 5;
		CentiSegments[i].pt.x = -1;
		CentiSegments[i].pt.y = -1;
		CentiSegments[i].nFlipCode = 1;
		if(cxDelta < 0)
			CentiSegments[i].bIsLeftSegment = TRUE;
		else
			CentiSegments[i].bIsLeftSegment = FALSE;
	}

	CentiSegments[nSegmentsInCentipede - 1].bIsHead = TRUE;

	if(cxDelta < 0)
	{
		// Moving left
		if(bMoveDownScreen)
		{
			xFirstSegment = rWin.right - CENTISEGMENTCX;
			yFirstSegment = rWin.top + 10;
		}
		else
		{
			xFirstSegment = rWin.right - CENTISEGMENTCX;
			yFirstSegment = rWin.bottom - CENTISEGMENTCY - 10;
		}
	}
	else
	{
		// Moving right
		if(bMoveDownScreen)
		{
			xFirstSegment = rWin.left;
			yFirstSegment = rWin.top + 10;
		}
		else
		{
			xFirstSegment = rWin.left - CENTISEGMENTCX;
			yFirstSegment = rWin.bottom - CENTISEGMENTCY - 10;
		}
	}

	x = xFirstSegment;
	y = yFirstSegment;

	bDownScreen = bMoveDownScreen;
	nWiggleCode = 0;
}


// Used for copying out 'cels' or individual centipede segments from
// single bitmap containing all centipede segments.
const int HEADCELSX = 1;
const int HEADCELSY = 3;

const int BODYCELSX = 1;
const int BODYCELSY = 47;

BOOL SimCentipede::GetFirstCentipedeSpriteSegment(POINT& pt, LPRECT prXOR, LPRECT prAND)
{
	int Left, Top, Right, Bottom;

	if(!bInitOk)
		return FALSE;
	nSegmentCount = 0;

	pt.x = CentiSegments[nSegmentCount].pt.x;
	pt.y = CentiSegments[nSegmentCount].pt.y;

	// Copy out position of correct 'cel' in Centipede cel bitmaps.
	if(CentiSegments[nSegmentCount].bIsLeftSegment)
		Left = HEADCELSX + (CentiSegments[nSegmentCount].nWiggleCode - 1) * CENTISEGMENTCX;
	else
		Left = HEADCELSX + (5 * CENTISEGMENTCX) + (CentiSegments[nSegmentCount].nWiggleCode - 1) * CENTISEGMENTCX;

	if(CentiSegments[nSegmentCount].bIsHead)
		Top = HEADCELSY;
	else
		Top = BODYCELSY;

	Right = Left + CENTISEGMENTCX;
	Bottom = Top + CENTISEGMENTCY;

	SetRect(prXOR, Left, Top, Right, Bottom);
	SetRect(prAND, Left, Top, Right, Bottom);

	return TRUE;
}


BOOL SimCentipede::GetNextCentipedeSpriteSegment(POINT& pt, LPRECT prXOR, LPRECT prAND)
{
	int Left, Top, Right, Bottom;

	if(!bInitOk || nSegmentCount >= nSegmentsInCentipede - 1)
		return FALSE;

	nSegmentCount++;

	// Copy out position of correct 'cel' in Centipede cel bitmaps.
	if(CentiSegments[nSegmentCount].bIsLeftSegment)
		Left = HEADCELSX + (CentiSegments[nSegmentCount].nWiggleCode - 1) * CENTISEGMENTCX;
	else
		Left = HEADCELSX + (5 * CENTISEGMENTCX) + (CentiSegments[nSegmentCount].nWiggleCode - 1) * CENTISEGMENTCX;

	if(CentiSegments[nSegmentCount].bIsHead)
		Top = HEADCELSY;
	else
		Top = BODYCELSY;

	Right = Left + CENTISEGMENTCX;
	Bottom = Top + CENTISEGMENTCY;

	SetRect(prXOR, Left, Top, Right, Bottom);
	SetRect(prAND, Left, Top, Right, Bottom);

	pt.x = CentiSegments[nSegmentCount].pt.x;
	pt.y = CentiSegments[nSegmentCount].pt.y;

	return TRUE;
}


int SimCentipede::GetHitsForSegmentPos(int xTest, int yTest)
{
	if(!bInitOk)
		return UNKNOWN;

	int nHits = 0;

	if(xTest < 0)
		nHits |= LEFTSCREEN;
	if(xTest + CENTISEGMENTCX > rWin.right)
		nHits |= RIGHTSCREEN;
	if(yTest < 0)
		nHits |= TOPSCREEN;
	if(yTest + CENTISEGMENTCY > rWin.bottom)
		nHits |= BOTTOMSCREEN;

	return nHits;
}


void SimCentipede::UpdateSegmentArray(void)
{
	//int bIsLeft = cxDelta < 0 ? TRUE : FALSE;
	if(!bInitOk)
		return;

	int i;

	int xNext;
	int yNext;

	int xLast;
	int yLast;

	int nLastPosCode = cxDelta < 0 ? -1 : 1;  // 1 = Normal, 0 straight down or up, -1 reverse

	int bWasFlipped;

	int rc;

	// CENTISEGMENTINFO contains:
	// 	POINT pt;  // Upper left position of this segment
	// 	int bIsHead;
	// 	int nWiggleCode; // 1 to 5
	// 	int nFlipCode; // -1, 0 or 1
	// 	int bIsLeftSegment;

	// Walks through segment array and calculates new positions
	// based on initial (X, Y)
	//if(bFirstTime)
	//{
	xLast = x;
	yLast = y;

	for(i = 0; i < nSegmentsInCentipede; i++)
	{
		xNext = xLast + CENTIMOVECX * nLastPosCode;
		yNext = yLast;

		bWasFlipped = FALSE;

		rc = GetHitsForSegmentPos(xNext, yNext);
		if(rc)
		{
			// Then we've bumped into something.
			// Advance a level up or down and switch direction.
			xNext = xLast;
			if(bDownScreen)
			{
				yNext = yNext + CENTIMOVECY;
				rc = GetHitsForSegmentPos(xNext, yNext);
				if(rc && BOTTOMSCREEN)
					yNext = yNext - (CENTIMOVECY * 2);
			}
			else
			{
				yNext = yNext - CENTIMOVECY;
				rc = GetHitsForSegmentPos(xNext, yNext);
				if(rc && TOPSCREEN)
					yNext = yNext + (CENTIMOVECY * 2);
			}
			nLastPosCode *= -1;  // We've been flipped.
			bWasFlipped = TRUE;
		}
		//
		// 	POINT pt;  // Upper left position of this segment
		// 	int bIsHead;
		// 	int nWiggleCode; // 1 to 5
		// 	int nFlipCode; // -1, 0 or 1
		// 	int bIsLeftSegment;
		if(i == nSegmentsInCentipede - 1)
			CentiSegments[i].bIsHead = TRUE;
		else
			CentiSegments[i].bIsHead = FALSE;

		CentiSegments[i].pt.x = xNext;
		CentiSegments[i].pt.y = yNext;
		if(nWiggleCode == 0)
		{
			// 'Wiggle' legs between centipede segments...
			if(i % 2 == 0)
				CentiSegments[i].nWiggleCode = 1;
			else
				CentiSegments[i].nWiggleCode = 5;
		}
		else if(nWiggleCode == 1)
		{
			if(i % 2 == 0)
				CentiSegments[i].nWiggleCode = 2;
			else
				CentiSegments[i].nWiggleCode = 4;
		}
		else if(nWiggleCode == 2)
		{
			if(i % 2 == 0)
				CentiSegments[i].nWiggleCode = 3;
			else
				CentiSegments[i].nWiggleCode = 3;
		}
		else if(nWiggleCode == 3)
		{
			if(i % 2 == 0)
				CentiSegments[i].nWiggleCode = 4;
			else
				CentiSegments[i].nWiggleCode = 2;
		}
		else
		{
			if(i % 2 == 0)
				CentiSegments[i].nWiggleCode = 5;
			else
				CentiSegments[i].nWiggleCode = 1;
		}

		CentiSegments[i].nFlipCode = bWasFlipped;

		CentiSegments[i].bIsLeftSegment = nLastPosCode < 0 ? TRUE : FALSE;

		xLast = xNext;
		yLast = yNext;
	}

	// Advance 'leg' positions for next time.
	nWiggleCode = ++nWiggleCode % 5;
}


void SimCentipede::CalcNewPos(void)
{
	int xNew = x + cxDelta;
	int yNew = y;

	int rc = GetHitsForSegmentPos(xNew, y);
	if(rc)
	{
		// Then we've bumped into something.
		// Advance a level up or down and switch direction.
		xNew = x;
		if(bDownScreen)
		{
			yNew = y + CENTIMOVECY;
			rc = GetHitsForSegmentPos(xNew, yNew);
			if(rc && BOTTOMSCREEN)
			{
				yNew = yNew - (CENTIMOVECY * 2);
				bDownScreen = FALSE;
			}
		}
		else
		{
			yNew = y - CENTIMOVECY;
			rc = GetHitsForSegmentPos(xNew, yNew);
			if(rc && TOPSCREEN)
			{
				yNew = yNew + (CENTIMOVECY * 2);
            bDownScreen = TRUE;
			}
		}
		cxDelta *= -1;
		// We've been flipped.
	}
	x = xNew;
	y = yNew;
}


const int CROCCXDELTA = 15;

int SimAlligator::Init(HWND hwndMyFrame)
{
	// Load our bitmaps and initialize.
	AddBitmapFrameToArray(0, 0,
								 "BMP_CROC_11",
								 "BMP_CROC_11_MASK",
								 CROCCXDELTA * 2, 0);  // ("Crooked tail" needs an CX boost for this frame.)
	AddBitmapFrameToArray(0, 1,
								 "BMP_CROC_12",
								 "BMP_CROC_12_MASK",
								 CROCCXDELTA, 0);
	AddBitmapFrameToArray(0, 2,
								 "BMP_CROC_13",
								 "BMP_CROC_13_MASK",
								 CROCCXDELTA, 0);
	AddBitmapFrameToArray(0, 3,
								 "BMP_CROC_14",
								 "BMP_CROC_14_MASK",
								 CROCCXDELTA, 0);

	AddBitmapFrameToArray(1, 0,
								 "BMP_CROC_21",
								 "BMP_CROC_21_MASK",
								 -CROCCXDELTA, 0);
	AddBitmapFrameToArray(1, 1,
								 "BMP_CROC_22",
								 "BMP_CROC_22_MASK",
								 -CROCCXDELTA, 0);
	AddBitmapFrameToArray(1, 2,
								 "BMP_CROC_23",
								 "BMP_CROC_23_MASK",
								 -CROCCXDELTA, 0);
	AddBitmapFrameToArray(1, 3,
								 "BMP_CROC_24",
								 "BMP_CROC_24_MASK",
								 -CROCCXDELTA, 0);
	SortBitmapFramesArray();

	int rc = LoadResourcesInFramesArray(hwndMyFrame);

	if(!rc)
		MessageBox(NULL, "Can't load Alligator Bitmaps!", "Can't Load All Bitmaps", MB_ICONSTOP);

	// Start out in center of the screen.
	x = (rWin.right - rWin.left) /2;
	y = (rWin.bottom - rWin.top) / 2;

	// Generate a random direction and 'cel'.
	nCurrentDirection = random(1);
	nCurrentFrame = random(4);
	return rc;
}


void SimAlligator::UpdateNewPosition(int bChangeFrameToo)
{
	// Call base class method first.
	SCartoon::UpdateNewPosition(bChangeFrameToo);

	int rc = IsOnEdge(rWin);

   // Reverse direction.
	if(rc == RIGHT)
	{
		nCurrentDirection = 1;
		nCurrentFrame = 0;
		x -= CROCCXDELTA;
	}
	else if(rc == LEFT)
	{
		nCurrentDirection = 0;
		nCurrentFrame = 0;
		x += CROCCXDELTA * 2;
	}
}




