/***************************************************************
	SIM.H -- Header file for sprites and a simulation control
				object.
***************************************************************/
#include "ssprite.h"
#include "scartoon.h"
#include "colors.h"
#include <stdlib.h>

class SimMushroom : public SSprite
{
public:
	SimMushroom(HWND hwnd = NULL, LPSTR lpszXORBmpName = NULL, LPSTR lpszANDBmpName = NULL, int xPos = NULL, int yPos = NULL);

	~SimMushroom() {};

	void InitPos(int xNew, int yNew);
};

// Hit-test codes.
const int LEFTSCREEN = 1;
const int TOPSCREEN = 2;
const int RIGHTSCREEN = 4;
const int BOTTOMSCREEN = 8;
const int UNKNOWN = -1;


// Size of each centipede 'segment'.
const int CENTISEGMENTCX = 26;
const int CENTISEGMENTCY = 38;

const int CENTIMOVECX = 21;
const int CENTIMOVECY = 21;

const int MAXCENTISEGMENTS = 12;  // Longest centipede we can handle.

typedef struct tagCENTISEGMENTINFO
{
	POINT pt;  // Upper left position of this segment
	int bIsHead;
	int nWiggleCode; // 1 to 5
	int nFlipCode; // -1, 0 or 1
	int bIsLeftSegment;
}  CENTISEGMENTINFO;


class SimCentipede : public SSprite
{
public:
	// Constructor
	SimCentipede(HWND hwnd = NULL, LPSTR lpszXORBmpName = NULL, LPSTR lpszANDBmpName = NULL);

	// In case we are using default constructor, do real initialization.
	int Init(HWND hwnd, LPSTR lpszXORBmpName, LPSTR lpszANDBmpName);

	// Destructor
	~SimCentipede(void);

	// Centipede is treated as a series of independent 'segments.'
	// Each is a copied into a Sprite object along with coordinates for display.
	BOOL GetFirstCentipedeSpriteSegment(POINT& pt, LPRECT prXOR, LPRECT prAND);
	BOOL GetNextCentipedeSpriteSegment(POINT& pt, LPRECT prXOR, LPRECT prAND);

	// Create a new centipede
	void InitNewCentipede(int cxDelta, int nSegments, int bMoveDownScreen = TRUE);

	// Calc. new position for last segment.
	void CalcNewPos(void);

	// Generate "wiggling" legs on segments by changing bitmaps and the positions
	// of each segment.
	void UpdateSegmentArray(void);

	// Can we display the next segment on-screen? Check for hits with Mushrooms, too.
	int GetHitsForSegmentPos(int xTest, int yTest);

	// Setting this enables testing on-screen
	void SetActiveScreen(HDC hdc) { hdcActive = hdc; }

	void SaveWindowSize(HWND hwnd);

private:
	int nSegmentCount;
	int nSegmentsInCentipede;

	int bDownScreen;  // FALSE is a centipede moving up-the-screen.
	int nWiggleCode;

	CENTISEGMENTINFO CentiSegments[MAXCENTISEGMENTS];

	RECT rWin;

	HDC hdcActive;

	int bInitOk;
};


class SimAlligator : public SCartoon
{
public:
	SimAlligator(HWND hwnd) : SCartoon(2, 4) { SetCurrentWindowDim(hwnd); }
	~SimAlligator() { }

	// Initialize a new alligator.
	int Init(HWND hwnd);

	// Calc. new position for alligator.
	virtual void UpdateNewPosition(int bChangeFrameToo = TRUE);
};

