/*********************************************************
	SLIST.CPP -- Simple linked-list classes used to assist
				  in the "simple" Windows class library.
				  (Note: the classes in this file are designed to
                     be simple, not C++ savvy.)
**********************************************************/
#define MSC_VER
#include "slist.h"

LINKEDPTRLIST::LINKEDPTRLIST(void)
{
	Current = NULL;
	Tail = NULL;

	nCount = 0;

	nErrCode = 0;
	_fmemset(szErrMsg, 0, 80);

	Head = new PTRNODE;
	if(!Head)
	{
		lstrcpy(szErrMsg, "Can't allocate Head record for list");
		nErrCode = ALLOC_FAILED;
		return;
	}
	else
	{
		Head->lpNext = (LPSTR)Tail;
	}
}

void LINKEDPTRLIST::EmptyList(void)
{
	PTRNODE *Navig = (PTRNODE *)Head->lpNext;
	PTRNODE *Next;
	LPSTR lpThisData;

	// Kill all nodes...
	while (Navig)
	{
		lpThisData = Navig->lpData;
		delete[] Navig->lpData;

		Next = (PTRNODE *)Navig->lpNext;

		delete Navig;

		Navig = Next;
	}

	Current = NULL;
	Head->lpNext = NULL;
	Tail = NULL;

	nCount = 0;

	lpThisData;  // Kill warning.
	Next;        // ditto.
}

int LINKEDPTRLIST::AddNode(LPSTR lpData, int nRecSize)
{
	if(!lpData)
	{
		lstrcpy(szErrMsg, "Null pointer for list object data!");
		nErrCode = NULL_POINTER;
		return FALSE;
	}

	LPSTR NewData = new char[nRecSize];
	if (!NewData)
	{
		lstrcpy(szErrMsg, "Can't allocate memory for data in list object.");
		nErrCode = ALLOC_FAILED;
		return FALSE;
	}

	PTRNODE *NewNode = new PTRNODE;
	if (!NewNode)
	{
		lstrcpy(szErrMsg, "Can't allocate memory for node in list object.");
		nErrCode = ALLOC_FAILED;
		delete[] NewData;
		return FALSE;
	}

	_fmemcpy(NewData, lpData, nRecSize);

	if(!Current)
	{
		// Then we're the first node on the block!
		Current = NewNode;
		Current->lpData = NewData;
		Head->lpNext = (LPSTR)Current;
		Current->lpNext = (LPSTR)Tail;
	}
	else
	{
		PTRNODE *OldNext = (PTRNODE *)Current->lpNext;
		Current->lpNext = (LPSTR)NewNode;
		NewNode->lpNext = (LPSTR)OldNext;
		Current = NewNode;

		Current->lpData = NewData;

		OldNext;  // Kill warning.
	}
	nCount++;

	return TRUE;
}


int LINKEDPTRLIST::AddPreexistingNode(LPSTR lpData)
{
	if(!lpData)
	{
		lstrcpy(szErrMsg, "Null pointer for list object data!");
		nErrCode = NULL_POINTER;
		return FALSE;
	}

	PTRNODE *NewNode = new PTRNODE;
	if (!NewNode)
	{
		lstrcpy(szErrMsg, "Can't allocate memory for node in list object.");
		nErrCode = ALLOC_FAILED;
		return FALSE;
	}

	if(!Current)
	{
		// Then we're the first node on the block!
		Current = NewNode;
		Current->lpData = lpData;
		Head->lpNext = (LPSTR)Current;
		Current->lpNext = (LPSTR)Tail;
	}
	else
	{
		PTRNODE *OldNext = (PTRNODE *)Current->lpNext;
		Current->lpNext = (LPSTR)NewNode;
		NewNode->lpNext = (LPSTR)OldNext;
		Current = NewNode;

		Current->lpData = lpData;

		OldNext;  // Kill warning.
	}
	nCount++;

	return TRUE;
}


int LINKEDPTRLIST::DeleteNode(int nIndex)
{
	PTRNODE *Prev;

	int i = -1;
   Prev = Head;
	for(PTRNODE *Navig = (PTRNODE *)Head->lpNext; Navig != NULL; Navig = (PTRNODE *)Navig->lpNext)
	{
		i++;
		if(i == nIndex)
			break;
		Prev = Navig;	 
	}
		 
	if(Navig)
	{
   	// Disconnect from list;
		PTRNODE *DelNode = Navig;
      Prev->lpNext = DelNode->lpNext;

		// Delete data and node itself.
		delete [] DelNode->lpData;
		delete DelNode;
		nCount--;
		return TRUE;
	}
	else
   	return FALSE;
}

void LINKEDPTRLIST::GetLastError(int& nErr, LPSTR lpszErrDesc)
{
	nErr = nErrCode;

	if(lpszErrDesc)
   	lstrcpy(lpszErrDesc, szErrMsg);
}

void LINKEDPTRLIST::ClearError(void)
{
	nErrCode = 0;
   _fmemset(szErrMsg, 0, 80);
}
