/*********************************************************
	SSPRITE.CPP-- Implementation for a sprite class.
*********************************************************/
#include "sbitmap.h"
#include "ssprite.h"

SSprite::SSprite(HWND hwnd, LPSTR lpszXORBitmap, LPSTR lpszANDBitmap, int xPos, int yPos, int cxNewDelta, int cyNewDelta)
{
	HINSTANCE hInst;

#if defined(WIN32)
	hInst = (HINSTANCE)GetWindowLong(hwnd, GWL_HINSTANCE);
#else
	hInst = GetWindowWord(hwnd, GWW_HINSTANCE);
#endif

	hbmXOR = NULL;
	hbmAND = NULL;

	x = 0;
	y = 0;

	cx = 0;
	cy = 0;

	cxDelta = 0;
	cyDelta = 0;

	xOld = -1;
	yOld = -1;

	x = xPos;
	y = yPos;

	cxDelta = cxNewDelta;
	cyDelta = cyNewDelta;

	if(!hInst || !lpszXORBitmap || !lpszANDBitmap)
		// Initialize later;
		return;
	else
		LoadBitmaps(hInst, lpszXORBitmap, lpszANDBitmap);
}


// Copy contructor
SSprite::SSprite(const SSprite& sprOld)
{
	hbmXOR = sprOld.hbmXOR;
	hbmAND = sprOld.hbmAND;

	x = sprOld.x;
	y = sprOld.y;

	cx = sprOld.cx;
	cy = sprOld.cy;

	cxDelta = sprOld.cxDelta;
	cyDelta = sprOld.cyDelta;

	xOld = sprOld.xOld;
	yOld = sprOld.yOld;
}


SSprite& SSprite::operator=(const SSprite& sprOld)
{
	if(this != &sprOld)
	{
		hbmXOR = sprOld.hbmXOR;
		hbmAND = sprOld.hbmAND;

		x = sprOld.x;
		y = sprOld.y;

		cx = sprOld.cx;
		cy = sprOld.cy;

		cxDelta = sprOld.cxDelta;
		cyDelta = sprOld.cyDelta;

		xOld = sprOld.xOld;
		yOld = sprOld.yOld;
	}
	return *this;
}


SSprite::~SSprite(void)
{
	if(hbmXOR)
		DeleteObject(hbmXOR);

	if(hbmAND)
		DeleteObject(hbmAND);
}


int SSprite::LoadBitmaps(HWND hwnd, LPSTR lpszXORBitmap, LPSTR lpszANDBitmap)
{
	HINSTANCE hInst;
#if defined(WIN32)
		hInst = (HINSTANCE)GetWindowLong(hwnd, GWL_HINSTANCE);
#else
		hInst = GetWindowWord((HWND)hwnd, GWW_HINSTANCE);
#endif

	char szMsg[256];
	BITMAP bm;

	hbmXOR = LoadBitmap(hInst, lpszXORBitmap);
	if(!hbmXOR)
	{
		wsprintf(szMsg, "Unable to load (XOR) resource bitmap '%s'.", lpszXORBitmap);
		MessageBox(NULL, szMsg, "Can't Load Bitmap", MB_ICONSTOP);
		return FALSE;
	}
	else
	{
		GetObject(hbmXOR, sizeof(BITMAP), (LPSTR)&bm);
		cx = bm.bmWidth;
		cy = bm.bmHeight;
	}

	hbmAND = LoadBitmap(hInst, lpszANDBitmap);
	if(!hbmAND)
	{
		wsprintf(szMsg, "Unable to load (AND) resource bitmap '%s'.", lpszANDBitmap);
		MessageBox(NULL, szMsg, "Can't Load Bitmap", MB_ICONSTOP);
		DeleteObject(hbmXOR);
		hbmXOR = NULL;
		return FALSE;
	}

	return TRUE;
}


void SSprite::UpdatePos(void)
{
	x += cxDelta;
	y += cyDelta;
	xOld = x;
	yOld = y;
}


// Check for hit-test of sprite on-edge of window.
// or area defined by prTest.
int SSprite::IsOnEdge(const RECT& rTest)
{
	RECT rSpriteDim;

	rSpriteDim.left = x;
	rSpriteDim.top = y;
	rSpriteDim.right = rSpriteDim.left + cx;  // Right edge of bitmap.
	rSpriteDim.bottom = rSpriteDim.top + cy;  // Bottom edge of bitmap.

	// Check to see if sprite touches edge of specified rectangular area,
	//	if so return appropriate edge code.
	if(rSpriteDim.left <= rTest.left)
		return LEFT;
	else if(rSpriteDim.top <= rTest.top)
		return TOP;
	else if(rSpriteDim.right >= rTest.right)
		return RIGHT;
	else if(rSpriteDim.bottom >= rTest.bottom)
		return BOTTOM;
	else
		// Otherwise, sprite is within client area.
		return FALSE;
}


void SSprite::ChangeDirection(eEdges EdgeHitCode)
{
	// Change direction here to simulate a pool ball bouncing off a cushion.
	int cxNewDelta = cxDelta;
	int cyNewDelta = cyDelta;

	if(cxNewDelta == 0 || cyNewDelta == 0) // Vector is perpendicular N, S, E or W;
								  // Reverse direction appropriately.
	{
		cxNewDelta = -cxNewDelta;
		cyNewDelta = -cyNewDelta;
		cxDelta = cxNewDelta;
		cyDelta = cyNewDelta;
		return;
	}
	else if(cxNewDelta > 0 && cyNewDelta < 0)  // Vector is NE
	{
		if(EdgeHitCode == TOP)
		{
			// Change vector to SE
			cyNewDelta = -cyNewDelta;
			cyDelta = cyNewDelta;
			return;
		}
		else // EdgeHitCode == RIGHT
		{
			// Change vector to NW
			cxNewDelta = -cxNewDelta;
			cxDelta = cxNewDelta;
			return;
		}
	}
	else if(cxNewDelta > 0 && cyNewDelta > 0)  // Vector is SE
	{
		if(EdgeHitCode == RIGHT)
		{
			// Change vector to SW
			cxNewDelta = -cxNewDelta;
			cxDelta = cxNewDelta;
			return;
		}
		else // EdgeHitCode == BOTTOM
		{
			// Change vector to NE
			cyNewDelta = -cyNewDelta;
			cyDelta = cyNewDelta;
			return;
		}
	}
	else if(cxNewDelta < 0 && cyNewDelta > 0) // Vector is SW
	{
		if(EdgeHitCode == BOTTOM)
		{
			// Change vector to NW
			cyNewDelta = -cyNewDelta;
			cyDelta = cyNewDelta;
			return;
		}
		else // EdgeHitCode == LEFT
		{
			// Change vector to NE
			cxNewDelta = -cxNewDelta;
			cxDelta = cxNewDelta;
			return;
		}
	}
	else // cxNewDelta < 0 && cyNewDelta < 0 -- Vector is NW
	{
		if(EdgeHitCode == LEFT)
		{
			// Change vector to NE
			cxNewDelta = -cxNewDelta;
			cxDelta = cxNewDelta;
			return;
		}
		else // EdgeHitCode == TOP
		{
			// Change vector to SW
			cyNewDelta = -cyNewDelta;
			cyDelta = cyNewDelta;
			return;
		}
	}
}




