/*********************************************************
	SWINDOWS.H -- Header file for "simple" Windows
					  class library.

					  Designed to allow (beginning) non-Windows
					  C++ programmers to get at Windows
					  functionality without all the API/MFC/OWL
					  overhead.

**********************************************************/
#if !defined(_SWINDOWS_H)
	#define _SWINDOWS_H
#include <windows.h>
#if defined(MSC_VER)
	#include "msc_fix"
#endif
#include "serrs.h"
#include "slist.h"
#include "sdrawcor.h"
#include "sdraw.h"
#include "sdialog.h"

#if defined (_MSC_VER)
	#include "msc_fix.h"
#else
	#include <mem.h>
#endif

// You should provide a Main() function.
// WinMain() is the actual entry point into the program.
extern void Main(void);
extern char szDefaultAppName[];

class SWindows
{
public:
	// Constructor
	SWindows();

	// Destructor
	virtual ~SWindows();

	// Can be accessed by anyone.
	SDraw *pGraphics;

	HWND hwnd;
	HMENU hmenu;
	HICON hIconApp;
	HINSTANCE hInst;
	HINSTANCE hPrevInst;

	char szCaption[80];
	char szCommandLine[128];
	int nShowCommand;
	char szDefMenuName[40];

	int X, Y, CX, CY;

	int bMessageHandled;

	virtual int InitApp(HINSTANCE hInst, HINSTANCE hPrevInst, LPSTR lpszCmdLine, int nCmdShow, UINT uWinStyle = NULL);
	virtual int InitMainWindow(DWORD dwWinStyle = NULL);

	virtual void DoMessageLoop(void);

	virtual void RunIt(void);

	LRESULT MainWndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);

	// Use these two methods to use SDraw GDI graphics in your program.
	BOOL InitGraphics(HWND hwnd, int bUseOffscreenBuffering = FALSE);
	void CleanupGraphics(void);

	virtual void OnCreate(HWND hwnd, LPARAM lParam);
	virtual void OnSize(int cxSize, int cySize);

	virtual void OnInitMenu(){};
	virtual void OnCommand(int nCommand);

	virtual void OnDraw(SDraw& Graphics){};
	virtual void OnPaint(SDraw& Graphics){ bMessageHandled = FALSE; };

	virtual void OnEvKey(UINT uKey){};
	virtual void OnEvChar(UINT uChar){};
	virtual void OnTimer(int nTimerId){};
	virtual void OnIdleEvent(HWND hwnd, SDraw& Graphics){};
	virtual void OnDestroy();

	// void RequestDblClicks(void);

	virtual void OnLButtonDown(POINT ptMouse){};
	virtual void OnLButtonUp(POINT ptMouse){};
	virtual void OnLButtonDblClick(POINT ptMouse){};

	virtual void OnRButtonDown(POINT ptMouse){};
	virtual void OnRButtonUp(POINT ptMouse){};
	virtual void OnRButtonDblClick(POINT ptMouse){};

	virtual void OnMouseMove(POINT ptMouse){};

	// Command events are 'friends' outside
	// the class, but get passed an object
	// so they can used SWindows methods.
	friend void CmFileExit(SWindows *pObj);
	friend void CmHelpAbout(SWindows *pObj);

	// Misc. Window methods.
	void SetHInstance(HINSTANCE hMyInst) { hInst = hMyInst; };
	void SetHPrevInstance(HINSTANCE hMyPrevInst) { hPrevInst = hMyPrevInst; };
	void SetCommandLine(LPSTR lpszCmdLine) { if(lpszCmdLine) lstrcpy(szCommandLine, lpszCmdLine); else _fmemset(szCommandLine, 0, 128); };
	void SetShowWindowCommand(int nShowCmd) { nShowCommand = nShowCmd; };

	void SetCaption(LPSTR lpszCaption = NULL);
	void SetIcon(HICON hIcon);
	void SetIcon(LPSTR lpszIconResourceName);
	void ShowWaitCursor(void);
	void ShowArrowCursor(void);

	void RepaintWindow(void);
	void MoveWindow(int nX, int nY, int nCX, int nCY, int bRedraw = TRUE);
	void CloseApp(void);

	int MsgBox(LPSTR lpszText, LPSTR lpszCaption = szDefaultAppName, int nOptions = MB_ICONEXCLAMATION);

#define INCL_DEBUG
#if defined (INCL_DEBUG)
	void Dbg(LPSTR lpszStringVal, LPSTR lpszLabel = "Debug");
	void Dbg(int nVal, LPSTR lpszLabel = "Debug");
	void Dbg(UINT uVal, LPSTR lpszLabel = "Debug");
	void Dbg(LONG lVal, LPSTR lpszLabel = "Debug");
	void Dbg(DWORD dwVal, LPSTR lpszLabel = "Debug");
#endif

	// Menu methods.
	void SetMenu(LPSTR lpszMenuName);

	int AddMenuItemEventHandler(int nMenuItemId, HANDLERPROC lpfnHandler);
	void ClearMenuItemEventHandlers(void);

	void CheckMenuItem(int nMenuItemId);
	void CheckMenuItem(int nMenuItemId, int bIsChecked);
	void UncheckMenuItem(int nMenuItemId);

	void EnableMenuItem(int nMenuItemId);
	void DisableMenuItem(int nMenuItemId);
	void EnableMenuItem(int nMenuItemId, int bEnabled);  // (Emulates API call.)
	void SetMenuItemPrompt(int nMenuItemId, LPSTR lpszMenuText);

	// Keystroke methods.
	//int AddKeyStrokeEventHandler(int nKeyId, HANDLERPROC lpfnHandler);
	//int AddKeyStrokeEventHandler(char chKey, HANDLERPROC lpfnHandler);
	//void ClearKeyStrokeEventHandlers(void);

	// Timer methods.
	int StartTimer(int nTimerId, UINT uMilliSeconds, HANDLERPROC lpfnTimerProc = NULL);
	int StopTimer(int nTimerId);

	void StopAllTimers(void);
	int AddTimerToList(int nTimerId);

	// Add 'idle-time handler' -- use for simulations, etc.
	void SetIdleTimeEventHandler(HANDLERPROC lpfnHandler);
	void ClearIdleTimeEventHandler(void);

	// Mouse methods.
	//int SetLeftMouseButtonDownHandler(HANDLERPROC lpfnHandler);
	//int SetLeftMouseButtonUpHandler(HANDLERPROC lpfnHandler);

	//int SetRightMouseButtonDownHandler(HANDLERPROC lpfnHandler);
	//int SetRightMouseButtonUpHandler(HANDLERPROC lpfnHandler);

	//int SetMouseMoveHandler(HANDLERPROC lpfnHandler);

	//int ClearMouseEventHandlers(void);

	// 'About' dialog box methods.
	void SetAboutBoxMenuItem(int nMenuId, HANDLERPROC lpfnAboutHandler);
	void EnableAboutBox();
	void DisableAboutBox();
	void SetAboutBoxPrompts(LPSTR lpszLine1 = "About this program.", LPSTR lpszLine2 = NULL, LPSTR lpszLine3 = NULL, LPSTR lpszCaption = "About");
	// REM void SetAboutBoxIcon(LPSTR lpszAboutBoxIconName = NULL);

	// Error routines -- Structured exception handling
	//                   will be added in a later version.
	int GetLastError(LPSTR lpszErrDesc = NULL);
	LPSTR GetErrMsg(void) {return szErrMsg; };
	void SetError(int nErr, LPSTR lpszErrMsg);
	void ClearError(void);

private:
	int bMenuNeedsDestroying;

	int bEnableAboutBox;
	char szDefaultAboutBoxText[80];
	char szAboutBoxLine1Text[80];
	char szAboutBoxLine2Text[80];
	char szAboutBoxLine3Text[80];
	char szAboutBoxCaption[80];
	// REM char szAboutBoxIconName[40];

	LINKEDPTRLIST *MenuItemHandlerList;

	//LINKEDPTRLIST *KeyPressHandlerList;

	LINKEDPTRLIST *TimerList;

	HANDLERPROC IdleEventHandler;

	int nErrCode;
	char szErrMsg[80];
};

#endif
